@@ -4,18 +4,18 @@
 
 #include "artefact-prop-type.h"
 #include "beam-type.h"
+#include "caught-type.h"
 #include "conduct-type.h"
 #include "constrict-type.h"
 #include "energy-use-type.h"
-#include "equipment-type.h"
 #include "god-type.h"
 #include "item-prop-enum.h"
 #include "kill-method-type.h"
 #include "mon-holy-type.h"
+#include "movement-type.h"
 #include "random-var.h"
 #include "ouch.h"
 #include "pronoun-type.h"
-#include "reach-type.h"
 #include "size-part-type.h"
 #include "size-type.h"
 #include "stat-type.h"
@@ -44,29 +44,23 @@ public:
     virtual god_type  deity() const = 0;
 
     virtual bool      alive() const = 0;
+    virtual bool      alive_or_reviving() const = 0;
 
     // Should return false for perma-summoned things.
-    virtual bool is_summoned(int* duration = nullptr,
-                             int* summon_type = nullptr) const = 0;
+    virtual bool is_summoned() const = 0;
+    virtual bool was_created_by(int summon_type) const = 0;
+    virtual bool was_created_by(const actor& summoner,
+                                int summon_type = SPELL_NO_SPELL) const = 0;
 
-    virtual bool is_perm_summoned() const = 0;
+    virtual bool is_firewood() const = 0;
+    virtual bool is_peripheral() const = 0;
 
-    // [ds] Low-level moveto() - moves the actor without updating relevant
-    // grids, such as env.mgrid.
-    virtual void moveto(const coord_def &c, bool clear_net = true) = 0;
-
-    // High-level actor movement. If in doubt, use this. Returns false if the
-    // actor cannot be moved to the target, possibly because it is already
-    // occupied.
-    virtual bool move_to_pos(const coord_def &c, bool clear_net = true,
-                             bool force = false) = 0;
-
-    virtual void apply_location_effects(const coord_def &oldpos,
-                                        killer_type killer = KILL_NONE,
-                                        int killernum = -1) = 0;
-
-    // Handles sticky flame / barbs on deliberate movement.
-    virtual void did_deliberate_movement() = 0;
+    virtual bool move_to(const coord_def& newpos, movement_type flags = MV_DEFAULT,
+                         bool defer_finalisation = false) = 0;
+    virtual void finalise_movement(const actor* to_blame = nullptr) = 0;
+    void trigger_movement_effects(movement_type mvflags = MV_DEFAULT,
+                                  const actor* to_blame = nullptr);
+    void clear_deferred_move();
 
     virtual void set_position(const coord_def &c);
     const coord_def& pos() const { return position; }
@@ -87,55 +81,40 @@ public:
     virtual int       get_hit_dice() const = 0;
     virtual int       get_experience_level() const = 0;
 
-    virtual bool shove(const char* feat_name = "") = 0;
     virtual bool can_pass_through_feat(dungeon_feature_type grid) const = 0;
     virtual bool can_pass_through(int x, int y) const;
     virtual bool can_pass_through(const coord_def &c) const;
     virtual bool can_burrow() const = 0;
 
     virtual bool is_habitable_feat(dungeon_feature_type actual_grid) const = 0;
-            bool is_habitable(const coord_def &pos) const;
+    virtual bool is_habitable(const coord_def &pos) const;
 
     virtual size_type body_size(size_part_type psize = PSIZE_TORSO,
                                 bool base = false) const = 0;
 
-    virtual brand_type damage_brand(int which_attack = -1) = 0;
-    virtual vorpal_damage_type damage_type(int which_attack = -1) = 0;
     virtual item_def *weapon(int which_attack = -1) const = 0;
     const item_def *primary_weapon() const
     {
         return weapon(0);
     }
     virtual item_def *offhand_weapon() const { return nullptr; }
-    virtual random_var attack_delay(const item_def *projectile = nullptr,
-                                    bool rescale = true) const = 0;
+    virtual random_var attack_delay(const item_def *projectile = nullptr) const = 0;
+    virtual random_var melee_attack_delay() const = 0;
     virtual int has_claws(bool allow_tran = true) const = 0;
     virtual item_def *shield() const = 0;
-    virtual item_def *slot_item(equipment_type eq,
-                                bool include_melded=false) const = 0;
-    virtual int wearing(equipment_type slot, int sub_type) const = 0;
-    virtual int wearing_ego(equipment_type slot, int sub_type) const = 0;
+    virtual item_def *offhand_item() const = 0;
+    virtual item_def *body_armour() const = 0;
+    virtual int wearing(object_class_type obj_type, int sub_type,
+                        bool count_plus = 0, bool check_attuned = false) const = 0;
+    virtual int wearing_ego(object_class_type obj_type, int ego) const = 0;
+    int wearing_jewellery(int sub_type) const;
     virtual int scan_artefacts(artefact_prop_type which_property,
                                vector<const item_def *> *matches = nullptr) const = 0;
+    virtual bool unrand_equipped(int unrand_index, bool include_melded = false) const = 0;
 
     virtual hands_reqd_type hands_reqd(const item_def &item,
                                        bool base = false) const;
 
-            bool can_wield(const item_def* item,
-                           bool ignore_curse = false,
-                           bool ignore_brand = false,
-                           bool ignore_shield = false,
-                           bool ignore_transform = false) const;
-    virtual bool can_wield(const item_def &item,
-                           bool ignore_curse = false,
-                           bool ignore_brand = false,
-                           bool ignore_shield = false,
-                           bool ignore_transform = false) const = 0;
-    virtual bool could_wield(const item_def &item,
-                             bool ignore_brand = false,
-                             bool ignore_transform = false,
-                             bool quiet = true) const = 0;
-
     virtual string name(description_level_type type,
                         bool force_visible = false,
                         bool force_article = false) const = 0;
@@ -155,7 +134,7 @@ public:
     virtual bool can_see_invisible() const = 0;
     virtual bool invisible() const = 0;
     virtual bool nightvision() const = 0;
-    virtual reach_type reach_range() const = 0;
+    virtual int reach_range(bool include_weapon = true) const = 0;
 
     // Would looker be able to see the actor when in LOS?
     virtual bool visible_to(const actor *looker) const = 0;
@@ -180,8 +159,9 @@ public:
     virtual bool has_blood(bool temp = true) const = 0;
     virtual bool has_bones(bool temp = true) const = 0;
     virtual bool is_stationary() const = 0;
-    virtual bool malmutate(const string &reason) = 0;
-    virtual bool polymorph(int pow, bool allow_immobile = true) = 0;
+    virtual bool malmutate(const actor* source, const string &reason = "") = 0;
+    virtual bool polymorph(int dur) = 0;
+    virtual bool doom(int amount) = 0;
     virtual bool drain(const actor *agent, bool quiet = false,
                        int pow = 3) = 0;
     virtual int  hurt(const actor *attacker, int amount,
@@ -193,8 +173,8 @@ public:
                       bool attacker_effects = true) = 0;
     virtual bool heal(int amount) = 0;
     virtual void banish(const actor *agent, const string &who = "",
-                        const int power = 0, bool force = false) = 0;
-    virtual void blink() = 0;
+                        bool force = false) = 0;
+    virtual void blink(bool ignore_stasis = false) = 0;
     virtual void teleport(bool right_now = false,
                           bool wizard_tele = false) = 0;
     virtual bool poison(actor *attacker, int amount = 1, bool force = false) = 0;
@@ -203,26 +183,31 @@ public:
                           string source = "") = 0;
     virtual void petrify(const actor *attacker, bool force = false) = 0;
     virtual bool fully_petrify(bool quiet = false) = 0;
+    virtual bool vex(const actor* who, int dur, string source = "", string special_msg = "") = 0;
     virtual void slow_down(actor *attacker, int strength) = 0;
     virtual void confuse(actor *attacker, int strength) = 0;
-    virtual void put_to_sleep(actor *attacker, int strength,
+    virtual void put_to_sleep(actor *attacker, int duration,
                               bool hibernate = false) = 0;
     virtual void weaken(const actor *attacker, int pow) = 0;
+    virtual void diminish(const actor *attacker, int pow) = 0;
     virtual bool strip_willpower(actor *attacker, int dur,
                                  bool quiet = false) = 0;
+    virtual bool drain_magic(actor *attacker, int pow) = 0;
+    virtual void daze(int duration) = 0;
+    virtual void vitrify(const actor *attacker, int duration, bool quiet = false) = 0;
+    virtual bool floodify(const actor *attacker, int duration, const char* substance = "water") = 0;
     virtual void expose_to_element(beam_type element, int strength = 0,
+                                   const actor* source = nullptr,
                                    bool slow_cold_blood = true) = 0;
-    virtual void drain_stat(stat_type /*stat*/, int /*amount*/) { }
     virtual void splash_with_acid(actor *evildoer) = 0;
-    virtual void acid_corrode(int acid_strength) = 0;
-    virtual bool corrode_equipment(const char* corrosion_source = "the acid",
-                                   int degree = 1) = 0;
+    virtual bool corrode(const actor* source = nullptr,
+                         const char* corrosion_msg = "the acid",
+                         int amount = 4) = 0;
     virtual bool resists_dislodge(string /*event*/ = "") const { return false; };
 
     virtual bool can_hibernate(bool holi_only = false,
                                bool intrinsic_only = false) const;
     virtual bool can_sleep(bool holi_only = false) const;
-    virtual void check_awaken(int disturbance) = 0;
     virtual int beam_resists(bolt &beam, int hurted, bool doEffects,
                              string source = "") = 0;
     virtual bool can_feel_fear(bool include_unknown) const = 0;
@@ -240,11 +225,8 @@ public:
 
     virtual bool can_throw_large_rocks() const = 0;
 
-    virtual bool can_be_dazzled() const = 0;
-    virtual bool can_be_blinded() const = 0;
-
     virtual int armour_class() const = 0;
-    virtual int gdr_perc() const = 0;
+    virtual int gdr_perc(bool random = true) const = 0;
     int apply_ac(int damage, int max_damage = 0,
                  ac_type ac_rule = ac_type::normal,
                  bool for_real = true) const;
@@ -256,7 +238,7 @@ public:
     virtual int shield_bonus() const = 0;
     virtual int shield_bypass_ability(int tohit) const = 0;
     virtual void shield_block_succeeded(actor *attacker);
-    virtual bool missile_repulsion() const = 0;
+    virtual int missile_repulsion() const = 0;
 
     virtual monster_type mons_species(bool zombie_base = false) const = 0;
 
@@ -269,7 +251,8 @@ public:
     virtual int  how_chaotic(bool check_spells_god = false) const = 0;
     virtual bool is_unbreathing() const = 0;
     virtual bool is_insubstantial() const = 0;
-    virtual int res_acid() const = 0;
+    virtual bool is_amorphous() const = 0;
+    virtual int res_corr() const = 0;
     virtual bool res_damnation() const = 0;
     virtual int res_fire() const = 0;
     virtual int res_steam() const = 0;
@@ -286,15 +269,16 @@ public:
     virtual bool res_polar_vortex() const = 0;
     virtual bool res_petrify(bool temp = true) const = 0;
     virtual bool res_constrict() const = 0;
+    virtual int res_blind() const = 0;
     int get_res(int res) const;
     virtual int willpower() const = 0;
     virtual int check_willpower(const actor* source, int power) const;
     virtual bool no_tele(bool blink = false, bool temp = true) const = 0;
     virtual int inaccuracy() const;
     int inaccuracy_penalty() const;
+    virtual int slaying(bool throwing = false, bool random = true) const = 0;
     virtual bool antimagic_susceptible() const = 0;
 
-    virtual bool res_corr(bool /*allow_random*/ = true, bool temp = true) const;
     bool has_notele_item(vector<const item_def *> *matches = nullptr) const;
     virtual bool stasis() const = 0;
     virtual bool cloud_immune(bool items = true) const;
@@ -305,6 +289,7 @@ public:
     virtual bool no_cast(bool items = true) const;
     virtual bool reflection(bool items = true) const;
     virtual int extra_harm(bool items = true) const;
+    virtual bool sunder_is_ready() const = 0;
 
     virtual bool rmut_from_item() const;
     virtual bool evokable_invis() const;
@@ -312,22 +297,30 @@ public:
     // Return an int so we know whether an item is the sole source.
     virtual int equip_flight() const;
     virtual int spirit_shield(bool items = true) const;
-    virtual bool rampaging() const;
+    virtual int rampaging() const;
 
     virtual bool is_banished() const = 0;
     virtual bool is_web_immune() const = 0;
     virtual bool is_binding_sigil_immune() const = 0;
     virtual bool airborne() const = 0;
-    virtual bool ground_level() const;
 
     virtual bool is_dragonkind() const;
     virtual int  dragon_level() const;
 
     virtual bool paralysed() const = 0;
+    virtual bool cannot_move() const = 0;
     virtual bool cannot_act() const = 0;
+    virtual bool helpless() const = 0;
     virtual bool confused() const = 0;
-    virtual bool caught() const = 0;
     virtual bool asleep() const { return false; }
+    virtual bool is_silenced() const = 0;
+
+    virtual bool caught() const = 0;
+    caught_type  caught_by() const;
+    virtual void struggle_against_net() = 0;
+    virtual bool trap_in_web() = 0;
+    virtual bool trap_in_net(bool real, bool quiet = false) = 0;
+    virtual void stop_being_caught(bool drop_net = false) = 0;
 
     // self_halo: include own halo (actually if self_halo = false
     //            and has a halo, returns false; so if you have a
@@ -386,48 +379,51 @@ public:
     // Constriction stuff:
 
     mid_t constricted_by;
+    constrict_type constricted_type;
     int escape_attempts;
 
     // mids of all actors we are constricting.
     // Freed and set to nullptr when empty.
     vector<mid_t> *constricting;
 
-    void start_constricting(actor &whom);
+    // Volatile state related to movement finalisation (should not be serialised)
+    bool move_needs_finalisation;
+    coord_def last_move_pos;
+    movement_type last_move_flags;
+
+    void start_constricting(actor &whom, constrict_type type, int duration = 0);
 
     void stop_constricting(mid_t whom, bool intentional = false,
                            bool quiet = false, const string& escape_verb = "");
     void stop_constricting_all(bool intentional = false, bool quiet = false);
-    void stop_directly_constricting_all(bool intentional = false,
-                                        bool quiet = false);
+    void stop_directly_constricting_all(bool entangling_only = false);
     void stop_being_constricted(bool quiet = false, const string& escape_verb = "");
 
     virtual bool attempt_escape() = 0;
 
     bool can_constrict(const actor &defender, constrict_type typ) const;
-    bool can_engulf(const actor &defender) const;
     bool has_invalid_constrictor(bool move = false) const;
     void clear_invalid_constrictions(bool move = false);
     void handle_constriction();
     bool is_constricted() const;
-    constrict_type get_constrict_type() const;
     bool is_constricting() const;
     bool is_constricting(const actor &victim) const;
-    int num_constricting() const;
+    int num_constricting(constrict_type type = CONSTRICT_MELEE) const;
     virtual bool has_usable_tentacle() const = 0;
     virtual int constriction_damage(constrict_type typ) const = 0;
-    virtual bool clear_far_engulf(bool force = false, bool moved = false) = 0;
 
     // Be careful using this, as it doesn't keep the constrictor in sync.
-    void clear_constricted();
+    virtual void clear_constricted();
 
     string describe_props() const;
 
     string resist_margin_phrase(int margin) const;
 
     void collide(coord_def newpos, const actor *agent, int damage);
-    bool knockback(const actor &cause, int dist, int pow, string source_name);
+    bool knockback(const actor &cause, int dist, int pow, string source_name,
+                   coord_def source = coord_def());
     coord_def stumble_pos(coord_def targ) const;
-    void stumble_away_from(coord_def targ, string src = "");
+    bool stumble_away_from(coord_def targ, string src = "");
 
     static const actor *ensure_valid_actor(const actor *act);
     static actor *ensure_valid_actor(actor *act);
