@@ -33,7 +33,6 @@
 
 #define MAX_GHOST_DAMAGE     50
 #define MAX_GHOST_HP        400
-#define MAX_GHOST_EVASION    50
 
 // Pan lord AOE conjuration spell list.
 static spell_type search_order_aoe_conj[] =
@@ -97,7 +96,6 @@ static spell_type search_order_misc[] =
     SPELL_VITRIFY,
     SPELL_SLEEP,
     SPELL_MASS_CONFUSION,
-    SPELL_DRAIN_MAGIC,
     SPELL_PETRIFY,
     SPELL_POLYMORPH,
     SPELL_SLOW,
@@ -135,6 +133,7 @@ void ghost_demon::reset()
     max_hp           = 0;
     ev               = 0;
     ac               = 0;
+    willpower        = -1;
     damage           = 0;
     speed            = 10;
     move_energy      = 10;
@@ -147,6 +146,15 @@ void ghost_demon::reset()
     flies            = false;
     cloud_ring_ench  = ENCH_NONE;
     umbra_rad        = -1;
+    title            = "";
+}
+
+// Set values to the bare minimum required to function, in cases where full
+// setup will be done later.
+void ghost_demon::barebones_init()
+{
+    max_hp = 1;
+    xl = 1;
 }
 
 #define ADD_SPELL(which_spell) \
@@ -181,15 +189,11 @@ static attack_type _pan_lord_random_attack_type()
     attack_type attack = AT_HIT;
     if (one_chance_in(4))
     {
-        do
-        {
-            // An ugly list, but less brittle and without e.g. false trampling.
-            attack = static_cast<attack_type>(random_choose(AT_BITE, AT_STING,
-                                      AT_SPORE, AT_TOUCH, AT_PECK, AT_HEADBUTT,
-                                      AT_PUNCH, AT_KICK, AT_TENTACLE_SLAP,
-                                      AT_TAIL_SLAP, AT_GORE, AT_TRUNK_SLAP));
-        }
-        while (attack == AT_HIT || !is_plain_attack_type(attack));
+        // An ugly list, but less brittle and without e.g. false trampling.
+        attack = static_cast<attack_type>(random_choose(AT_BITE, AT_STING,
+                                    AT_SPORE, AT_TOUCH, AT_PECK, AT_HEADBUTT,
+                                    AT_PUNCH, AT_KICK, AT_TENTACLE_SLAP,
+                                    AT_TAIL_SLAP, AT_GORE, AT_TRUNK_SLAP));
     }
     return attack;
 }
@@ -228,9 +232,6 @@ void ghost_demon::set_pan_lord_special_attack()
         10, _brand_attack(SPWPN_VENOM),
         10, _brand_attack(SPWPN_DRAINING),
         2, _brand_attack(SPWPN_FOUL_FLAME),
-        4, _flavour_attack(AF_DRAIN_STR),
-        4, _flavour_attack(AF_DRAIN_INT),
-        2, _flavour_attack(AF_DRAIN_DEX),
         10, _flavour_attack(AF_DROWN),
         // Normal chance
         20, _brand_attack(SPWPN_FLAMING),
@@ -422,16 +423,13 @@ void ghost_demon::init_pandemonium_lord(bool friendly)
     colour = one_chance_in(10) ? colour_t{ETC_RANDOM} : random_monster_colour();
 }
 
-static const set<brand_type> ghost_banned_brands =
-                { SPWPN_HOLY_WRATH, SPWPN_CHAOS };
-
 void ghost_demon::init_player_ghost()
 {
     // don't preserve transformations for ghosty purposes
     unwind_var<transformation> form(you.form, transformation::none);
-    unwind_var<FixedBitVector<NUM_EQUIP>> melded(you.melded,
-                                                 FixedBitVector<NUM_EQUIP>());
+    unwind_var<player_equip_set> eq(you.equipment);
     unwind_var<bool> fishtail(you.fishtail, false);
+    you.equipment.unmeld_all_equipment(true);
 
     name   = you.your_name;
     max_hp = min(get_real_hp(false, false), MAX_GHOST_HP);
@@ -447,13 +445,13 @@ void ghost_demon::init_player_ghost()
     // clones might lack innate rPois, copy it. pghosts don't care.
     set_resist(resists, MR_RES_POISON, player_res_poison());
     set_resist(resists, MR_RES_NEG, you.res_negative_energy());
-    set_resist(resists, MR_RES_ACID, player_res_acid());
+    set_resist(resists, MR_RES_CORR, player_res_corrosion());
     // multi-level for players, boolean as an innate monster resistance
     set_resist(resists, MR_RES_STEAM, player_res_steam() ? 1 : 0);
     set_resist(resists, MR_RES_MIASMA, you.res_miasma());
     set_resist(resists, MR_RES_PETRIFY, you.res_petrify());
 
-    move_energy = 10;
+    move_energy = player_movement_speed(false, false);
     speed       = 10;
 
     damage = 4;
@@ -478,8 +476,8 @@ void ghost_demon::init_player_ghost()
             {
                 brand = static_cast<brand_type>(get_weapon_brand(weapon));
 
-                // normalize banned weapon brands
-                if (ghost_banned_brands.count(brand) > 0)
+                // No holy weapons for ghosts.
+                if (brand == SPWPN_HOLY_WRATH)
                     brand = SPWPN_NORMAL;
 
                 // Don't copy ranged- or artefact-only brands (reaping etc.).
@@ -494,7 +492,7 @@ void ghost_demon::init_player_ghost()
                 case STAFF_FIRE: brand = SPWPN_FLAMING; break;
                 case STAFF_COLD: brand = SPWPN_FREEZING; break;
                 case STAFF_ALCHEMY: brand = SPWPN_VENOM; break;
-                case STAFF_DEATH: brand = SPWPN_PAIN; break;
+                case STAFF_NECROMANCY: brand = SPWPN_PAIN; break;
                 case STAFF_AIR: brand = SPWPN_ELECTROCUTION; break;
                 case STAFF_EARTH: brand = SPWPN_HEAVY; break;
                 default: ;
@@ -527,6 +525,7 @@ void ghost_demon::init_player_ghost()
     best_skill = ::best_skill(SK_FIRST_SKILL, SK_LAST_SKILL);
     best_skill_level = you.skills[best_skill];
     xl = you.experience_level;
+    title = player_title();
 
     flies = true;
 
@@ -885,6 +884,10 @@ static attack_flavour _ugly_thing_colour_to_flavour(colour_t u_colour)
         u_att_flav = AF_COLD;
         break;
 
+    case MAGENTA:
+        u_att_flav = AF_ANTIMAGIC;
+        break;
+
     default:
         break;
     }
@@ -941,12 +944,16 @@ void ghost_demon::init_ugly_thing(bool very_ugly, bool only_mutate,
 
     // Pick a compatible attack flavour for this colour.
     att_flav = _ugly_thing_colour_to_flavour(colour);
-    if (colour == MAGENTA)
-        damage = damage * 4 / 3; // +5 for uglies, +9 for v uglies
 
     // Pick a compatible resistance for this attack flavour.
     ugly_thing_add_resistance(false, att_flav);
 
+    // Not really a normal resistance, so handle it separately.
+    if (colour == MAGENTA)
+        willpower = ((very_ugly) ? 120 : 80);
+    else
+        willpower = -1;
+
     // If this is a very ugly thing, upgrade it properly.
     if (very_ugly)
         ugly_thing_to_very_ugly_thing();
@@ -972,7 +979,7 @@ static resists_t _ugly_thing_resists(bool very_ugly, attack_flavour u_att_flav)
         return MR_RES_FIRE * (very_ugly ? 2 : 1);
 
     case AF_ACID:
-        return MR_RES_ACID;
+        return MR_RES_CORR;
 
     case AF_POISON:
     case AF_POISON_STRONG:
@@ -1059,6 +1066,21 @@ void ghost_demon::init_inugami_from_player(int power)
     see_invis = true;
 }
 
+void ghost_demon::init_platinum_paragon(int power)
+{
+    const monster_type type = MONS_PLATINUM_PARAGON;
+    const monsterentry* stats = get_monster_data(type);
+
+    speed = stats->speed;
+    ev = stats->ev;
+    ac = stats->AC + div_rand_round(power, 10);
+    max_hp = 50 + div_rand_round(power, 2);
+    xl = 10 + div_rand_round(power, 10);
+    move_energy = stats->energy_usage.move;
+    see_invis = true;
+    damage = 10 + div_rand_round(power, 5);
+}
+
 // Used when creating ghosts: goes through and finds spells for the
 // ghost to cast. Death is a traumatic experience, so ghosts only
 // remember a few spells.
@@ -1098,7 +1120,7 @@ spell_type ghost_demon::translate_spell(spell_type spell) const
         return SPELL_BLINK;
 #endif
     case SPELL_SWIFTNESS:
-        return SPELL_SPRINT;
+        return SPELL_FLEETFOOT;
     case SPELL_CONFUSING_TOUCH:
         return SPELL_CONFUSE;
     case SPELL_CURSE_OF_AGONY:
@@ -1109,6 +1131,12 @@ spell_type ghost_demon::translate_spell(spell_type spell) const
         return SPELL_SUMMON_DRAGON;
     case SPELL_ELECTRIC_CHARGE:
         return SPELL_ELECTROLUNGE;
+    case SPELL_PERCUSSIVE_TEMPERING:
+        return SPELL_ALL_PURPOSE_TEMPERING;
+    case SPELL_FREEZING_CLOUD:
+        return SPELL_FREEZING_GUST;
+    case SPELL_NOXIOUS_BOG:
+        return SPELL_HURL_SLUDGE;
     default:
         break;
     }
@@ -1201,64 +1229,130 @@ bool ghost_demon::ghost_eligible()
         && ghosts_nosave.count(you.where_are_you) == 0;
 }
 
-bool debug_check_ghost(const ghost_demon &ghost)
+bool debug_check_ghost(const ghost_demon &ghost, string &err)
 {
     // Values greater than the allowed maximum or less then the
     // allowed minimum signalise bugginess.
     if (ghost.damage < 0 || ghost.damage > MAX_GHOST_DAMAGE)
+    {
+        err = make_stringf("Bad damage: %d", ghost.damage);
         return false;
+    }
+
     if (ghost.max_hp < 1 || ghost.max_hp > MAX_GHOST_HP)
+    {
+        err = make_stringf("Bad max HP: %d", ghost.max_hp);
         return false;
+    }
+
     if (ghost.xl < 1 || ghost.xl > 27)
+    {
+        err = make_stringf("Bad XL: %d", ghost.xl);
         return false;
+    }
+
     if (ghost.ev > MAX_GHOST_EVASION)
+    {
+        err = make_stringf("Bad EV: %d", ghost.ev);
         return false;
+    }
+
     if (get_resist(ghost.resists, MR_RES_ELEC) < 0)
+    {
+        err = make_stringf("Bad rElec: %d", get_resist(ghost.resists, MR_RES_ELEC));
         return false;
+    }
+
     if (ghost.brand < SPWPN_NORMAL || ghost.brand > MAX_GHOST_BRAND)
+    {
+        err = make_stringf("Bad brand: %d", ghost.brand);
         return false;
+    }
+
     if (!species::is_valid(ghost.species))
+    {
+        err = make_stringf("Bad species: %d", ghost.species);
         return false;
+    }
+
     if (!job_type_valid(ghost.job))
+    {
+        err = make_stringf("Bad job: %d", ghost.job);
         return false;
+    }
+
     if (ghost.best_skill < SK_FIGHTING || ghost.best_skill >= NUM_SKILLS)
+    {
+        err = make_stringf("Bad best skill: %d", ghost.best_skill);
         return false;
+    }
+
     if (ghost.best_skill_level < 0 || ghost.best_skill_level > 27)
+    {
+        err = make_stringf("Bad best skill level: %d", ghost.best_skill_level);
         return false;
+    }
+
     if (ghost.religion < GOD_NO_GOD || ghost.religion >= NUM_GODS)
+    {
+        err = make_stringf("Bad god: %d", ghost.religion);
         return false;
+    }
 
     if (ghost.brand == SPWPN_HOLY_WRATH)
+    {
+        err = "Has holy wrath";
         return false;
+    }
 
     // Ghosts don't get non-plain attack types and flavours.
     if (ghost.att_type != AT_HIT || ghost.att_flav != AF_PLAIN)
+    {
+        err = make_stringf("Bad attack type/flavour: %d/%d", ghost.att_type, ghost.att_flav);
         return false;
+    }
 
-    // Name validation.
-    if (!validate_player_name(ghost.name))
-        return false;
     // Many combining characters can come per every letter, but if there's
     // that much, it's probably a maliciously forged ghost of some kind.
-    if (ghost.name.length() > MAX_NAME_LENGTH * 10 || ghost.name.empty())
+    if (!validate_player_name(ghost.name)
+        || ghost.name.length() > MAX_NAME_LENGTH * 10
+        || ghost.name.empty())
+    {
+        err = "Invalid name";
         return false;
+    }
+
     if (ghost.name != trimmed_string(ghost.name))
+    {
+        err = "Mismatched name";
         return false;
+    }
 
     // Check for non-existing spells.
     for (const mon_spell_slot &slot : ghost.spells)
+    {
         if (slot.spell < 0 || slot.spell >= NUM_SPELLS)
+        {
+            err = make_stringf("Bad spell: %d", slot.spell);
             return false;
+        }
+    }
 
     return true;
 }
 
 // Sanity checks for some ghost values.
-bool debug_check_ghosts(vector<ghost_demon> &ghosts)
+bool debug_check_ghosts(vector<ghost_demon> &ghosts, string &err)
 {
     for (const ghost_demon &ghost : ghosts)
-        if (!debug_check_ghost(ghost))
+    {
+        if (!debug_check_ghost(ghost, err))
+        {
+            err = make_stringf("Buggy ghost %s: %s", ghost.name.c_str(), err.c_str());
             return false;
+        }
+    }
+
     return true;
 }
 
