@@ -502,7 +502,56 @@ static int proc_mouse_event(int c, const MEVENT *me)
 }
 #endif
 
-static int pending = 0;
+static int curses_pending_key = 0;
+#ifdef USE_TILE_WEB
+static int tiles_pending_key = 0;
+#endif
+
+static bool _curses_fetch_pending_key(bool blocking)
+{
+    if (curses_pending_key)
+        return true;
+
+    wint_t c;
+    int i;
+    if (blocking)
+        i = get_wch(&c);
+    else
+    {
+        nodelay(stdscr, TRUE);
+        // apparently some need this to guarantee non-blocking -- bwr
+        timeout(0);
+        i = get_wch(&c);
+        nodelay(stdscr, FALSE);
+    }
+
+    switch (i)
+    {
+    case ERR:
+        // getch() returns -1 on EOF, convert that into an Escape. Evil hack,
+        // but the alternative is to explicitly check for -1 everywhere where
+        // we might otherwise spin in a tight keyboard input loop.
+        // XXX: this doesn't work with `blocking` false, because we can't tell
+        // a timeout from an error...
+        if (!blocking)
+            return false;
+        curses_pending_key = ESCAPE;
+        return true;
+    case OK:
+        // a normal (printable) key
+        curses_pending_key = c;
+        return true;
+    case KEY_CODE_YES:
+    default:
+        curses_pending_key = -c;
+        return true;
+    }
+}
+
+static bool _curses_has_key()
+{
+    return _curses_fetch_pending_key(false);
+}
 
 static int _get_key_from_curses()
 {
@@ -512,57 +561,39 @@ static int _get_key_from_curses()
     watchdog();
 #endif
 
-    if (pending)
+#ifdef USE_TILE_WEB
+    refresh();
+    tiles.redraw();
+
+    if (tiles_pending_key)
     {
-        int c = pending;
-        pending = 0;
+        wint_t c = tiles_pending_key;
+        tiles_pending_key = 0;
         return c;
     }
 
-    wint_t c;
-
-#ifdef USE_TILE_WEB
-    refresh();
-
-    tiles.redraw();
-    tiles.await_input(c, true);
-
+    wint_t c = tiles.await_input(&_curses_has_key);
     if (c != 0)
         return c;
 #endif
 
-    switch (get_wch(&c))
-    {
-    case ERR:
-        // getch() returns -1 on EOF, convert that into an Escape. Evil hack,
-        // but the alternative is to explicitly check for -1 everywhere where
-        // we might otherwise spin in a tight keyboard input loop.
-        return ESCAPE;
-    case OK:
-        // a normal (printable) key
-        return c;
-    }
-
-    return -c;
+    _curses_fetch_pending_key(true);
+    int result = curses_pending_key;
+    curses_pending_key = 0;
+    return result;
 }
 
 #if defined(KEY_RESIZE) || defined(USE_UNIX_SIGNALS)
 static void unix_handle_resize_event()
 {
     crawl_state.last_winch = time(0);
-    if (crawl_state.waiting_for_command)
+    if (crawl_state.waiting_for_command || crawl_state.waiting_for_ui)
         handle_terminal_resize();
     else
         crawl_state.terminal_resized = true;
 }
 #endif
 
-static bool getch_returns_resizes;
-void set_getch_returns_resizes(bool rr)
-{
-    getch_returns_resizes = rr;
-}
-
 static int _headless_getchk()
 {
 #ifdef WATCHDOG
@@ -571,19 +602,17 @@ static int _headless_getchk()
     watchdog();
 #endif
 
-    if (pending)
+#ifdef USE_TILE_WEB
+    tiles.redraw();
+
+    if (tiles_pending_key)
     {
-        int c = pending;
-        pending = 0;
+        wint_t c = tiles_pending_key;
+        tiles_pending_key = 0;
         return c;
     }
 
-
-#ifdef USE_TILE_WEB
-    wint_t c;
-    tiles.redraw();
-    tiles.await_input(c, true);
-
+    wint_t c = tiles.await_input([]() { return false; });
     if (c != 0)
         return c;
 #endif
@@ -597,11 +626,9 @@ static int _headless_getch_ck()
     do
     {
         c = _headless_getchk();
-        // TODO: release?
-        // XX this should possibly sleep
-    } while (
-             ((c == CK_MOUSE_MOVE || c == CK_MOUSE_CLICK)
-                 && !crawl_state.mouse_enabled));
+    }
+    while ((c == CK_MOUSE_MOVE || c == CK_MOUSE_CLICK)
+               && !crawl_state.mouse_enabled);
 
     return c;
 }
@@ -642,13 +669,11 @@ int getch_ck()
             // then scroll down one line. To fix this, we always sync termsz:
             crawl_view.init_geometry();
 
-            if (!getch_returns_resizes)
-                continue;
+            continue;
         }
 #endif
 
-        // TODO: what else should be added to this?
-        switch (c)
+        switch (-c)
         {
         case 127:
         // 127 is ASCII DEL, which some terminals (all mac, some linux) use for
@@ -659,35 +684,49 @@ int getch_ck()
         // reliably does get mapped to KEY_DC by ncurses. Some background:
         //     https://invisible-island.net/xterm/xterm.faq.html#xterm_erase
         // (I've never found documentation for the mac situation.)
-        case -KEY_BACKSPACE: return CK_BKSP;
-        case -KEY_DC:    return CK_DELETE;
-        case -KEY_HOME:  return CK_HOME;
-        case -KEY_PPAGE: return CK_PGUP;
-        case -KEY_END:   return CK_END;
-        case -KEY_NPAGE: return CK_PGDN;
-        case -KEY_UP:    return CK_UP;
-        case -KEY_DOWN:  return CK_DOWN;
-        case -KEY_LEFT:  return CK_LEFT;
-        case -KEY_RIGHT: return CK_RIGHT;
-#ifdef KEY_RESIZE
-        case -KEY_RESIZE: return CK_RESIZE;
-#endif
-        case -KEY_BTAB:  return CK_SHIFT_TAB;
-        case -KEY_SDC:   return CK_SHIFT_DELETE;
-#ifdef TARGET_OS_MACOSX
-        // not sure what's up with this, no ncurses constant? defining it only
-        // for mac to be cautious
-        case -515:       return CK_CTRL_DELETE;
-#endif
+        case KEY_BACKSPACE: return CK_BKSP;
+        case KEY_IC:        return CK_INSERT;
+        case KEY_DC:        return CK_DELETE;
+        case KEY_HOME:      return CK_HOME;
+        case KEY_END:       return CK_END;
+        case KEY_PPAGE:     return CK_PGUP;
+        case KEY_NPAGE:     return CK_PGDN;
+        case KEY_UP:        return CK_UP;
+        case KEY_DOWN:      return CK_DOWN;
+        case KEY_LEFT:      return CK_LEFT;
+        case KEY_RIGHT:     return CK_RIGHT;
+        case KEY_BEG:       return CK_CLEAR;
 
-        // may or may not be defined depending on the terminal. Escape codes
-        // are the xterm convention, other terminals may do different things
-        // and ncurses may or may not handle them correctly.
-        case -KEY_SR:    return CK_SHIFT_UP;     // \033[1;2A
-        case -KEY_SLEFT: return CK_SHIFT_LEFT;   // \033[1;2D
-        case -KEY_SRIGHT: return CK_SHIFT_RIGHT; // \033[1;2C
-        case -KEY_SF:    return CK_SHIFT_DOWN;   // \033[1;2B
-        default:         return c;
+        case KEY_BTAB:      return CK_SHIFT_TAB;
+        case KEY_SDC:       return CK_SHIFT_DELETE;
+        case KEY_SHOME:     return CK_SHIFT_HOME;
+        case KEY_SEND:      return CK_SHIFT_END;
+        case KEY_SPREVIOUS: return CK_SHIFT_PGUP;
+        case KEY_SNEXT:     return CK_SHIFT_PGDN;
+        case KEY_SR:        return CK_SHIFT_UP;
+        case KEY_SF:        return CK_SHIFT_DOWN;
+        case KEY_SLEFT:     return CK_SHIFT_LEFT;
+        case KEY_SRIGHT:    return CK_SHIFT_RIGHT;
+
+        case KEY_A1:        return CK_NUMPAD_7;
+        case KEY_A3:        return CK_NUMPAD_9;
+        case KEY_B2:        return CK_NUMPAD_5;
+        case KEY_C1:        return CK_NUMPAD_1;
+        case KEY_C3:        return CK_NUMPAD_3;
+
+        // Undocumented ncurses control keycodes, here be dragons!!!
+        case 515:           return CK_CTRL_DELETE; // Mac
+        case 526:           return CK_CTRL_DELETE; // Linux
+        case 542:           return CK_CTRL_HOME;
+        case 537:           return CK_CTRL_END;
+        case 562:           return CK_CTRL_PGUP;
+        case 557:           return CK_CTRL_PGDN;
+        case 573:           return CK_CTRL_UP;
+        case 532:           return CK_CTRL_DOWN;
+        case 552:           return CK_CTRL_LEFT;
+        case 567:           return CK_CTRL_RIGHT;
+
+        default:            return c;
         }
     }
 }
@@ -751,7 +790,7 @@ static void unixcurses_defkeys()
     define_key("\033Oo", 1012); // / (may conflict with the above define?)
     define_key("\033OX", 1021); // =, at least on mac console
 
-#ifdef TARGET_OS_MACOSX
+# ifdef TARGET_OS_MACOSX
     // force some mappings for function keys that work on mac Terminal.app with
     // the default TERM value.
 
@@ -771,63 +810,12 @@ static void unixcurses_defkeys()
     check_define_key("\033b", -(CK_LEFT + CK_ALT_BASE));
     check_define_key("\033f", -(CK_RIGHT + CK_ALT_BASE));
     // (sadly, only left and right have modifiers by default on Terminal.app)
-#endif
+# endif
 #undef check_define_key
 
-    // variants. Ugly curses won't allow us to return the same code...
-    // TODO: the above comment seems to be wrong for current ncurses?
-    define_key("\033[1~", 1031); // Home
-    define_key("\033[4~", 1034); // End
-    define_key("\033[E",  1040); // center arrow
 #endif
 }
 
-int unixcurses_get_vi_key(int keyin)
-{
-    switch (-keyin)
-    {
-    // TODO: should use cio.h constants, but I'm too scared to change this
-    // function
-    // -1001..-1009: passed without change
-    case 1031: return -1007;
-    case 1034: return -1001;
-    case 1040: return -1005;
-
-    case KEY_HOME:   return -1007;
-    case KEY_END:    return -1001;
-    case KEY_DOWN:   return -1002;
-    case KEY_UP:     return -1008;
-    case KEY_LEFT:   return -1004;
-    case KEY_RIGHT:  return -1006;
-    case KEY_NPAGE:  return -1003;
-    case KEY_PPAGE:  return -1009;
-    case KEY_A1:     return -1007;
-    case KEY_A3:     return -1009;
-    case KEY_B2:     return -1005;
-    case KEY_C1:     return -1001;
-    case KEY_C3:     return -1003;
-    case KEY_SHOME:  return 'Y';
-    case KEY_SEND:   return 'B';
-    case KEY_SLEFT:  return 'H';
-    case KEY_SRIGHT: return 'L';
-    case KEY_BTAB:   return CK_SHIFT_TAB;
-    case KEY_BACKSPACE:
-        // If terminfo's entry for backspace (kbs) is ctrl-h, curses
-        // generates KEY_BACKSPACE for the ctrl-h key. Work around that by
-        // converting back to CK_BKSP.
-        // Note that this mangling occurs entirely on the machine Crawl runs
-        // on (and even within crawl's process) rather than where the user's
-        // terminal is, so this check is reliable.
-        static char kbskey[] = "kbs"; // tigetstr wants a non-const pointer :(
-        static const char * const kbs = tigetstr(kbskey);
-        static const int bskey = (kbs && kbs != (const char *) -1
-                                      && kbs == string("\010")) ? CK_BKSP
-                                                                : KEY_BACKSPACE;
-        return bskey;
-    }
-    return keyin;
-}
-
 // Certain terminals support vt100 keypad application mode only after some
 // extra goading.
 #define KPADAPP "\033[?1051l\033[?1052l\033[?1060l\033[?1061h"
@@ -880,13 +868,13 @@ void console_startup()
 #ifdef CURSES_USE_KEYPAD
     keypad(stdscr, TRUE);
 
-#ifdef CURSES_SET_ESCDELAY
-#ifdef NCURSES_REENTRANT
+# ifdef CURSES_SET_ESCDELAY
+#  ifdef NCURSES_REENTRANT
     set_escdelay(CURSES_SET_ESCDELAY);
-#else
+#  else
     ESCDELAY = CURSES_SET_ESCDELAY;
-#endif
-#endif
+#  endif
+# endif
 #endif
 
     meta(stdscr, TRUE);
@@ -1863,21 +1851,18 @@ void delay(unsigned int time)
 
 static bool _headless_kbhit()
 {
-    // TODO: ??
-    if (pending)
-        return true;
-
 #ifdef USE_TILE_WEB
-    wint_t c;
-    bool result = tiles.await_input(c, false);
-
-    if (result && c != 0)
-        pending = c;
-
-    return result;
-#else
-    return false;
+    if (tiles_pending_key)
+        return true;
+    wint_t c = tiles.try_await_input();
+    if (c != 0)
+    {
+        tiles_pending_key = c;
+        return true;
+    }
 #endif
+
+    return false;
 }
 
 /* This is Juho Snellman's modified kbhit, to work with macros */
@@ -1886,35 +1871,19 @@ bool kbhit()
     if (_headless_mode)
         return _headless_kbhit();
 
-    if (pending)
+    if (_curses_has_key())
         return true;
 
-    wint_t c;
-#ifndef USE_TILE_WEB
-    int i;
-
-    nodelay(stdscr, TRUE);
-    timeout(0);  // apparently some need this to guarantee non-blocking -- bwr
-    i = get_wch(&c);
-    nodelay(stdscr, FALSE);
-
-    switch (i)
+#ifdef USE_TILE_WEB
+    if (tiles_pending_key)
+        return true;
+    wint_t c = tiles.try_await_input();
+    if (c != 0)
     {
-    case OK:
-        pending = c;
+        tiles_pending_key = c;
         return true;
-    case KEY_CODE_YES:
-        pending = -c;
-        return true;
-    default:
-        return false;
     }
-#else
-    bool result = tiles.await_input(c, false);
-
-    if (result && c != 0)
-        pending = c;
-
-    return result;
 #endif
+
+    return false;
 }
