@@ -87,6 +87,12 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_RAVEN,
+      {
+        { SPELL_ILL_OMEN, 24, MON_SPELL_NATURAL | MON_SPELL_NOISY },
+      }
+    },
+
     // ('c') Centaurs and such.
     {  MST_FAUN,
       {
@@ -133,7 +139,7 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_SLOW, 12, MON_SPELL_WIZARD },
        { SPELL_CONFUSE, 12, MON_SPELL_WIZARD },
-       { SPELL_REPEL_MISSILES, 12, MON_SPELL_WIZARD },
+       { SPELL_DEFLECT_MISSILES, 12, MON_SPELL_WIZARD },
        { SPELL_BLINK_RANGE, 36, MON_SPELL_WIZARD | MON_SPELL_SHORT_RANGE },
       }
     },
@@ -143,7 +149,7 @@ static const mon_spellbook mspell_list[] =
        { SPELL_HOARFROST_CANNONADE, 33, MON_SPELL_WIZARD },
        { SPELL_FIREBALL, 11, MON_SPELL_WIZARD },
        { SPELL_LRD, 33, MON_SPELL_WIZARD },
-       { SPELL_REPEL_MISSILES, 11, MON_SPELL_WIZARD },
+       { SPELL_DEFLECT_MISSILES, 11, MON_SPELL_WIZARD },
        { SPELL_HASTE, 11, MON_SPELL_WIZARD },
       }
     },
@@ -202,6 +208,14 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_ALDERKING,
+      {
+       { SPELL_GRASPING_ROOTS, 30, MON_SPELL_NATURAL },
+       { SPELL_SIPHON_ESSENCE, 25, MON_SPELL_MAGICAL },
+       { SPELL_DISPEL_UNDEAD, 15, MON_SPELL_MAGICAL },
+      }
+    },
+
     // ('g') Small humanoids.
     {  MST_BOUDA,
       {
@@ -220,6 +234,21 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_IRONBOUND_MECHANIST,
+      {
+       { SPELL_PHALANX_BEETLE, 35, MON_SPELL_WIZARD },
+       { SPELL_ALL_PURPOSE_TEMPERING, 45, MON_SPELL_WIZARD },
+      }
+    },
+
+    {  MST_SPROZZ,
+      {
+       { SPELL_CLOCKWORK_BEE, 70, MON_SPELL_WIZARD },
+       { SPELL_ALL_PURPOSE_TEMPERING, 30, MON_SPELL_WIZARD },
+      }
+    },
+
+
     // ('h') Carnivorous quadrupeds.
     {  MST_BEAR,
       {
@@ -251,9 +280,9 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
-    {  MST_DOOM_HOUND,
+    {  MST_OBLIVION_HOUND,
       {
-       { SPELL_DOOM_HOWL, 30, MON_SPELL_NATURAL },
+       { SPELL_OBLIVION_HOWL, 30, MON_SPELL_NATURAL },
       }
     },
 
@@ -279,7 +308,7 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_LIGHTNING_BOLT, 16, MON_SPELL_WIZARD },
        { SPELL_AIRSTRIKE, 32, MON_SPELL_WIZARD },
-       { SPELL_REPEL_MISSILES, 16, MON_SPELL_WIZARD },
+       { SPELL_DEFLECT_MISSILES, 16, MON_SPELL_WIZARD },
       }
     },
 
@@ -300,7 +329,7 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_WIND_BLAST, 50, MON_SPELL_NATURAL | MON_SPELL_BREATH },
        { SPELL_AIRSTRIKE, 29, MON_SPELL_NATURAL },
-       { SPELL_REPEL_MISSILES, 29, MON_SPELL_NATURAL },
+       { SPELL_DEFLECT_MISSILES, 29, MON_SPELL_NATURAL },
       }
     },
 
@@ -316,6 +345,13 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_WYRMHOLE,
+      {
+       { SPELL_SOJOURNING_BOLT, 50, MON_SPELL_NATURAL | MON_SPELL_BREATH
+                                        | MON_SPELL_NOISY },
+      }
+    },
+
     // ('l') Lizards.
     {  MST_BASILISK,
       {
@@ -323,10 +359,16 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_SEISMOSAURUS,
+      {
+       { SPELL_SEISMIC_STOMP, 56, MON_SPELL_NATURAL },
+      }
+    },
+
     // ('m') Merfolk and friends.
     {  MST_MERFOLK_SIREN,
       {
-       { SPELL_SIREN_SONG, 160, MON_SPELL_WIZARD },
+       { SPELL_SIREN_SONG, 80, MON_SPELL_WIZARD },
       }
     },
 
@@ -338,7 +380,7 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_MERFOLK_AVATAR,
       {
-       { SPELL_AVATAR_SONG, 160, MON_SPELL_WIZARD },
+       { SPELL_AVATAR_SONG, 80, MON_SPELL_WIZARD },
       }
     },
 
@@ -359,6 +401,12 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_COGNITOGAUNT,
+      {
+       { SPELL_BRAIN_BITE, 76, MON_SPELL_MAGICAL },
+      }
+    },
+
     // ('o') Orcs.
     {  MST_ORC_PRIEST,
       {
@@ -441,7 +489,7 @@ static const mon_spellbook mspell_list[] =
     {  MST_DEATH_KNIGHT,
       {
        { SPELL_AGONY, 15, MON_SPELL_PRIEST },
-       { SPELL_INJURY_MIRROR, 20, MON_SPELL_PRIEST },
+       { SPELL_HURL_TORCHLIGHT, 30, MON_SPELL_PRIEST },
       }
     },
 
@@ -467,6 +515,12 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_IRONBOUND_BEASTMASTER,
+      {
+       { SPELL_THROW_BOLAS, 40, MON_SPELL_WIZARD },
+      }
+    },
+
     {  MST_IRONBOUND_CONVOKER,
       {
        { SPELL_WORD_OF_RECALL, 30, MON_SPELL_WIZARD },
@@ -566,7 +620,7 @@ static const mon_spellbook mspell_list[] =
     // ('r') rodents
     {  MST_CRYSTAL_ECHIDNA,
       {
-       { SPELL_CRYSTALLIZING_SHOT, 30, MON_SPELL_NATURAL },
+       { SPELL_CRYSTALLISING_SHOT, 30, MON_SPELL_NATURAL },
        { SPELL_THROW_BARBS, 30, MON_SPELL_NATURAL },
       }
     },
@@ -616,6 +670,12 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_MOUNTAINSHELL,
+      {
+       { SPELL_LANDBREAKER, 50, MON_SPELL_NATURAL },
+      }
+    },
+
     // ('v') Vortices.
     {  MST_THERMIC_DYNAMO,
       {
@@ -676,12 +736,25 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_SCREAMING_REFRACTION,
+      {
+       { SPELL_INJURY_MIRROR, 40, MON_SPELL_MAGICAL },
+       { SPELL_LRD, 40, MON_SPELL_MAGICAL },
+      }
+    },
+
     { MST_BUNYIP,
       {
         { SPELL_WARNING_CRY, 20, MON_SPELL_VOCAL | MON_SPELL_BREATH },
       }
     },
 
+    { MST_CHONCHON,
+      {
+        { SPELL_BECKONING_GALE, 26, MON_SPELL_WIZARD },
+      }
+    },
+
     // ('y') Flying insects.
     {  MST_MELIAI,
       {
@@ -711,7 +784,7 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_GHOST_MOTH,
       {
-       { SPELL_DRAINING_GAZE, 100, MON_SPELL_MAGICAL | MON_SPELL_INSTANT },
+       { SPELL_ANTIMAGIC_GAZE, 100, MON_SPELL_MAGICAL | MON_SPELL_INSTANT },
       }
     },
 
@@ -775,6 +848,12 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_FRAVASHI,
+      {
+       { SPELL_SUMMON_MORTAL_CHAMPION, 64, MON_SPELL_MAGICAL },
+      }
+    },
+
     {  MST_SERAPH,
       {
        { SPELL_WARNING_CRY, 50, MON_SPELL_VOCAL| MON_SPELL_BREATH },
@@ -798,6 +877,12 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_CLOCKROACH,
+      {
+       { SPELL_SANDBLAST, 30, MON_SPELL_MAGICAL},
+      }
+    },
+
     {  MST_RADROACH,
       {
        { SPELL_IRRADIATE, 50, MON_SPELL_NATURAL},
@@ -928,6 +1013,12 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_BONE_DRAGON,
+      {
+       { SPELL_MARCH_OF_SORROWS, 59, MON_SPELL_NATURAL | MON_SPELL_NOISY },
+      }
+    },
+
     {  MST_PEARL_DRAGON,
       {
        { SPELL_HOLY_BREATH, 68, MON_SPELL_NATURAL | MON_SPELL_BREATH
@@ -957,6 +1048,12 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_MONARCH_BOMB,
+      {
+       { SPELL_DEPLOY_BOMBLET, 100, MON_SPELL_NATURAL},
+      }
+    },
+
     // ('G') Organs.
     {  MST_GLASS_EYE,
       {
@@ -971,15 +1068,21 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_EYE_OF_DRAINING,
+      {
+       { SPELL_DRAINING_GAZE, 100, MON_SPELL_MAGICAL | MON_SPELL_INSTANT },
+      }
+    },
+
     {  MST_EYE_OF_DEVASTATION,
       {
-       { SPELL_BOLT_OF_DEVASTATION, 60, MON_SPELL_MAGICAL },
+       { SPELL_BOLT_OF_DEVASTATION, 90, MON_SPELL_MAGICAL },
       }
     },
 
     {  MST_SHINING_EYE,
       {
-       { SPELL_MALMUTATE, 57, MON_SPELL_MAGICAL },
+       { SPELL_MUTAGENIC_GAZE, 150, MON_SPELL_MAGICAL | MON_SPELL_INSTANT },
       }
     },
 
@@ -994,7 +1097,6 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_GLOWING_ORANGE_BRAIN,
       {
-       { SPELL_BRAIN_BITE, 14, MON_SPELL_MAGICAL },
        { SPELL_CAUSE_FEAR, 14, MON_SPELL_MAGICAL },
        { SPELL_SHADOW_CREATURES, 14, MON_SPELL_MAGICAL },
        { SPELL_MASS_CONFUSION, 14, MON_SPELL_MAGICAL },
@@ -1008,24 +1110,37 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_HIPPOGRIFF,
+      {
+        { SPELL_WARNING_CRY, 35, MON_SPELL_VOCAL | MON_SPELL_BREATH },
+        { SPELL_BECKONING_GALE, 60, MON_SPELL_NATURAL },
+      }
+    },
+
     { MST_FENSTRIDER_WITCH,
       {
        { SPELL_PARALYSE, 15, MON_SPELL_WIZARD },
-       { SPELL_AGONIZING_TOUCH, 30, MON_SPELL_WIZARD },
+       { SPELL_AGONISING_TOUCH, 30, MON_SPELL_WIZARD },
        { SPELL_HURL_SLUDGE, 15, MON_SPELL_WIZARD },
       }
     },
 
-    {  MST_SPHINX,
+    {  MST_SPHINX_MARAUDER,
       {
+       { SPELL_CONFUSE, 14, MON_SPELL_WIZARD },
+       { SPELL_VEX, 22, MON_SPELL_WIZARD },
+      }
+    },
+
+    {  MST_GUARDIAN_SPHINX,
+      {
        { SPELL_CONFUSE, 10, MON_SPELL_WIZARD },
-       { SPELL_SLOW, 10, MON_SPELL_WIZARD },
-       { SPELL_PARALYSE, 10, MON_SPELL_WIZARD },
-       { SPELL_SMITING, 10, MON_SPELL_WIZARD },
+       { SPELL_SLOW, 14, MON_SPELL_WIZARD },
+       { SPELL_PARALYSE, 14, MON_SPELL_WIZARD },
+       { SPELL_SMITING, 14, MON_SPELL_WIZARD },
       }
     },
 
-    // ('H') Hybrids.
     {  MST_JOROGUMO,
       {
        { SPELL_BOLT_OF_DRAINING, 20, MON_SPELL_WIZARD },
@@ -1038,7 +1153,7 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_BOLT_OF_COLD, 20, MON_SPELL_MAGICAL },
        { SPELL_THROW_ICICLE, 20, MON_SPELL_MAGICAL },
-       { SPELL_FREEZING_CLOUD, 20, MON_SPELL_MAGICAL },
+       { SPELL_FREEZING_GUST, 20, MON_SPELL_MAGICAL },
        { SPELL_SUMMON_ICE_BEAST, 40, MON_SPELL_MAGICAL },
       }
     },
@@ -1054,7 +1169,7 @@ static const mon_spellbook mspell_list[] =
     {  MST_ORANGE_CRYSTAL_STATUE,
       {
        { SPELL_SHADOW_CREATURES, 50, MON_SPELL_MAGICAL },
-       { SPELL_DRAINING_GAZE, 25, MON_SPELL_MAGICAL },
+       { SPELL_ANTIMAGIC_GAZE, 25, MON_SPELL_MAGICAL },
        { SPELL_MASS_CONFUSION, 25, MON_SPELL_MAGICAL },
       }
     },
@@ -1066,6 +1181,12 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_BOUNDLESS_TESSERACT,
+      {
+       { SPELL_PLANAR_OVERLAY, 0, MON_SPELL_NATURAL },
+      }
+    },
+
     // ('K') Kobolds.
     {  MST_KOBOLD_DEMONOLOGIST,
       {
@@ -1075,12 +1196,25 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_KOBOLD_GEOMANCER,
+      {
+       { SPELL_LRD, 32, MON_SPELL_WIZARD },
+      }
+    },
+
     {  MST_KOBOLD_BLASTMINER,
       {
        { SPELL_BOMBARD, 32, MON_SPELL_NATURAL | MON_SPELL_NOISY },
       }
     },
 
+    {  MST_KOBOLD_FLESHCRAFTER,
+      {
+       { SPELL_BOLT_OF_FLESH, 32, MON_SPELL_WIZARD },
+       { SPELL_AWAKEN_FLESH, 60, MON_SPELL_WIZARD },
+      }
+    },
+
     // ('L') Liches and the like.
     {  MST_REVENANT,
       {
@@ -1091,6 +1225,13 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_NAMELESS_REVENANT,
+      {
+       { SPELL_PYRRHIC_RECOLLECTION, 0, MON_SPELL_NATURAL },
+       { SPELL_BLINK_CLOSE, 15, MON_SPELL_WIZARD},
+      }
+    },
+
     {  MST_LICH,
       {
        { SPELL_BOLT_OF_COLD, 12, MON_SPELL_WIZARD },
@@ -1124,7 +1265,7 @@ static const mon_spellbook mspell_list[] =
       {
        {
         { SPELL_WIND_BLAST, 20, MON_SPELL_WIZARD },
-        { SPELL_STOKE_FLAMES, 10, MON_SPELL_WIZARD },
+        { SPELL_STOKE_FLAMES, 20, MON_SPELL_WIZARD },
        }
       }
     },
@@ -1212,7 +1353,7 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
-    // ('O') Ogres.
+    // ('O') Ogres and oni.
     {  MST_OGRE_MAGE,
       {
        { SPELL_BOLT_OF_MAGMA, 15, MON_SPELL_WIZARD },
@@ -1229,6 +1370,13 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_ONI_INCARCERATOR,
+      {
+       { SPELL_MANIFOLD_ASSAULT, 40, MON_SPELL_WIZARD },
+       { SPELL_THROW_BOLAS, 12, MON_SPELL_WIZARD },
+      }
+    },
+
     // ('P') Immobile plants.
     {  MST_SCRUB_NETTLE,
       {
@@ -1259,7 +1407,8 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_BATTLESPHERE, 30, MON_SPELL_WIZARD },
        { SPELL_LIGHTNING_BOLT, 20, MON_SPELL_WIZARD },
-       { SPELL_FORCE_LANCE, 10, MON_SPELL_WIZARD | MON_SPELL_EMERGENCY },
+       { SPELL_FORCE_LANCE, 10, MON_SPELL_WIZARD },
+       { SPELL_FORCE_LANCE, 20, MON_SPELL_WIZARD | MON_SPELL_EMERGENCY },
       }
     },
 
@@ -1271,7 +1420,7 @@ static const mon_spellbook mspell_list[] =
         { SPELL_CORROSIVE_BOLT, 11, MON_SPELL_WIZARD },
         { SPELL_LIGHTNING_BOLT, 11, MON_SPELL_WIZARD },
         { SPELL_BOLT_OF_DRAINING, 11, MON_SPELL_WIZARD },
-        { SPELL_REPEL_MISSILES, 11, MON_SPELL_WIZARD },
+        { SPELL_DEFLECT_MISSILES, 11, MON_SPELL_WIZARD },
       }
     },
 
@@ -1306,6 +1455,12 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_NEKOMATA,
+      {
+       { SPELL_PHANTOM_BLITZ, 48, MON_SPELL_MAGICAL },
+      }
+    },
+
     {  MST_WENDIGO,
       {
        { SPELL_SERACFALL, 50, MON_SPELL_MAGICAL },
@@ -1377,7 +1532,22 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_VAMPIRE_BLOODPRINCE,
+      {
+       { SPELL_GRAVE_CLAW, 30, MON_SPELL_WIZARD },
+       { SPELL_DRAIN_LIFE, 30, MON_SPELL_WIZARD },
+       { SPELL_RAVENOUS_SWARM, 40, MON_SPELL_WIZARD },
+       { SPELL_DOMINATE_UNDEAD, 22, MON_SPELL_WIZARD },
+      }
+    },
+
     // ('W') Incorporeal undead.
+    {  MST_BES_KEMWAR,
+      {
+       { SPELL_DIMINISH_SPELLS, 44, MON_SPELL_NATURAL },
+      }
+    },
+
     {  MST_SHADOWGHAST,
       {
        { SPELL_INVISIBILITY, 33, MON_SPELL_NATURAL },
@@ -1495,8 +1665,8 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_NEQOXEC,
       {
-       { SPELL_MALMUTATE, 14, MON_SPELL_MAGICAL },
-       { SPELL_BRAIN_BITE, 24, MON_SPELL_MAGICAL },
+       { SPELL_WARP_BODY, 30, MON_SPELL_MAGICAL },
+       { SPELL_BRAIN_BITE, 28, MON_SPELL_MAGICAL },
       }
     },
 
@@ -1526,22 +1696,30 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_BLIZZARD_DEMON,
       {
-       { SPELL_BOLT_OF_COLD, 16, MON_SPELL_MAGICAL },
-       { SPELL_LIGHTNING_BOLT, 16, MON_SPELL_MAGICAL },
-       { SPELL_FREEZING_CLOUD, 16, MON_SPELL_MAGICAL },
-       { SPELL_AIRSTRIKE, 16, MON_SPELL_MAGICAL },
+       { SPELL_LIGHTNING_BOLT, 21, MON_SPELL_MAGICAL },
+       { SPELL_FREEZING_GUST, 21, MON_SPELL_MAGICAL },
+       { SPELL_SLEETSTRIKE, 21, MON_SPELL_MAGICAL },
+       { SPELL_DEFLECT_MISSILES, 16, MON_SPELL_MAGICAL },
       }
     },
 
     {  MST_CACODEMON,
       {
-       { SPELL_BOLT_OF_DEVASTATION, 20, MON_SPELL_MAGICAL },
+       { SPELL_DOOM_BOLT, 22, MON_SPELL_MAGICAL },
        { SPELL_VITRIFY, 20, MON_SPELL_MAGICAL },
-       { SPELL_MALMUTATE, 13, MON_SPELL_MAGICAL },
+       { SPELL_SLOW, 13, MON_SPELL_MAGICAL },
        { SPELL_DIG, 13, MON_SPELL_MAGICAL },
       }
     },
 
+    {  MST_ZYKZYL,
+      {
+       { SPELL_MALMUTATE, 15, MON_SPELL_MAGICAL },
+       { SPELL_ENFEEBLE, 20, MON_SPELL_MAGICAL },
+       { SPELL_BOLT_OF_FLESH, 13, MON_SPELL_MAGICAL },
+      }
+    },
+
     {  MST_HELLION,
       {
        { SPELL_CALL_DOWN_DAMNATION, 57, MON_SPELL_MAGICAL },
@@ -1657,7 +1835,7 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_CRYSTAL_GUARDIAN,
       {
-       { SPELL_CRYSTALLIZING_SHOT, 50, MON_SPELL_MAGICAL },
+       { SPELL_CRYSTALLISING_SHOT, 50, MON_SPELL_MAGICAL },
       }
     },
 
@@ -1669,7 +1847,13 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_HELLFIRE_MORTAR,
       {
-       { SPELL_BOLT_OF_MAGMA, 200, MON_SPELL_MAGICAL },
+       { SPELL_MAGMA_BARRAGE, 200, MON_SPELL_MAGICAL },
+      }
+    },
+
+    {  MST_SHADOW_TURRET,
+      {
+       { SPELL_SHADOW_SHOT, 200, MON_SPELL_MAGICAL },
       }
     },
 
@@ -1686,6 +1870,12 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_UNDERTAKER,
+      {
+       { SPELL_BORGNJORS_VILE_CLUTCH, 36, MON_SPELL_MAGICAL  | MON_SPELL_LONG_RANGE },
+      }
+    },
+
     // ('*') Concentrated orbs.
     {  MST_WRETCHED_STAR,
       {
@@ -1702,6 +1892,29 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_ORB_OF_ENTROPY,
+      {
+       { SPELL_BOLT_OF_DRAINING, 32, MON_SPELL_MAGICAL },
+       { SPELL_ACID_BALL, 32, MON_SPELL_MAGICAL },
+       { SPELL_ILL_OMEN, 16, MON_SPELL_MAGICAL },
+      }
+    },
+
+    {  MST_ORB_OF_WINTER,
+      {
+       { SPELL_OZOCUBUS_REFRIGERATION, 32, MON_SPELL_MAGICAL },
+       { SPELL_FLASH_FREEZE, 32, MON_SPELL_MAGICAL },
+       { SPELL_OSTRACISE, 16, MON_SPELL_MAGICAL },
+      }
+    },
+
+
+    {  MST_SAWBLADE,
+      {
+        { SPELL_SHRED, 200, MON_SPELL_NATURAL },
+      }
+    },
+
     // (';') Walking Tomes.
     { MST_WALKING_TOME,
       {
@@ -1798,7 +2011,7 @@ static const mon_spellbook mspell_list[] =
        { SPELL_MASS_CONFUSION, 11, MON_SPELL_WIZARD },
        { SPELL_STRIP_WILLPOWER, 11, MON_SPELL_WIZARD },
        { SPELL_HASTE, 11, MON_SPELL_WIZARD },
-       { SPELL_REPEL_MISSILES, 11, MON_SPELL_WIZARD },
+       { SPELL_DEFLECT_MISSILES, 11, MON_SPELL_WIZARD },
       }
     },
 
@@ -1847,7 +2060,7 @@ static const mon_spellbook mspell_list[] =
        { SPELL_AIRSTRIKE, 23, MON_SPELL_WIZARD },
        { SPELL_SUMMON_SMALL_MAMMAL, 12, MON_SPELL_WIZARD },
        { SPELL_SLOW, 12, MON_SPELL_WIZARD },
-       { SPELL_SPRINT, 12, MON_SPELL_WIZARD },
+       { SPELL_FLEETFOOT, 12, MON_SPELL_WIZARD },
       }
     },
 
@@ -1982,6 +2195,18 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_CREEPING_PLASMODIUM,
+      {
+       { SPELL_LAUNCH_SPORANGIUM, 70, MON_SPELL_NATURAL },
+      }
+    },
+
+    {  MST_NASCENT_PLASMODIUM,
+      {
+       { SPELL_ACID_BALL, 70, MON_SPELL_NATURAL },
+      }
+    },
+
     // ('K') Kobolds.
     {  MST_SONJA,
       {
@@ -2052,7 +2277,7 @@ static const mon_spellbook mspell_list[] =
        { SPELL_LIGHTNING_BOLT, 14, MON_SPELL_WIZARD },
        { SPELL_AIRSTRIKE, 14, MON_SPELL_WIZARD },
        { SPELL_AIR_ELEMENTALS, 14, MON_SPELL_WIZARD },
-       { SPELL_REPEL_MISSILES, 14, MON_SPELL_WIZARD },
+       { SPELL_DEFLECT_MISSILES, 14, MON_SPELL_WIZARD },
       }
     },
 
@@ -2093,8 +2318,8 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_MOON_TROLL,
       {
-       { SPELL_CORROSIVE_BOLT, 50, MON_SPELL_WIZARD },
-       { SPELL_DAZZLING_FLASH, 50, MON_SPELL_NATURAL },
+       { SPELL_CORROSIVE_BOLT, 35, MON_SPELL_WIZARD },
+       { SPELL_LRD, 35, MON_SPELL_WIZARD },
       }
     },
 
@@ -2111,9 +2336,9 @@ static const mon_spellbook mspell_list[] =
     // ('5') Lesser demons.
     {  MST_GRINDER,
       {
-       { SPELL_PAIN, 19, MON_SPELL_MAGICAL },
-       { SPELL_PARALYSE, 19, MON_SPELL_MAGICAL },
-       { SPELL_BLINK, 19, MON_SPELL_MAGICAL },
+       { SPELL_PAIN, 17, MON_SPELL_MAGICAL },
+       { SPELL_PARALYSE, 17, MON_SPELL_MAGICAL },
+       { SPELL_BLINK, 21, MON_SPELL_MAGICAL },
       }
     },
 
@@ -2291,7 +2516,7 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_NORRIS,
       {
-       { SPELL_DRAINING_GAZE, 36, MON_SPELL_PRIEST },
+       { SPELL_ANTIMAGIC_GAZE, 36, MON_SPELL_PRIEST },
        { SPELL_PRIMAL_WAVE, 30, MON_SPELL_PRIEST },
       }
     },
@@ -2300,16 +2525,15 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_PLASMA_BEAM, 24, MON_SPELL_WIZARD },
        { SPELL_BOMBARD, 24, MON_SPELL_WIZARD },
-       { SPELL_SPELLFORGED_SERVITOR, 32, MON_SPELL_WIZARD },
+       { SPELL_SPELLSPARK_SERVITOR, 32, MON_SPELL_WIZARD },
       }
     },
 
-    {  MST_PSYCHE,
+    {  MST_CASSANDRA,
       {
-       { SPELL_CANTRIP, 12, MON_SPELL_WIZARD },
-       { SPELL_POLYMORPH, 12, MON_SPELL_WIZARD },
-       { SPELL_CHAIN_OF_CHAOS, 24, MON_SPELL_WIZARD },
-       { SPELL_INVISIBILITY, 12, MON_SPELL_WIZARD },
+       { SPELL_DOOMSAYING, 200, MON_SPELL_NATURAL },
+       { SPELL_DOOM_BOLT, 34, MON_SPELL_WIZARD },
+       { SPELL_FLEETFOOT, 24, MON_SPELL_WIZARD },
       }
     },
     // ('&', mostly) Demon lords.
@@ -2323,14 +2547,14 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_BOLT_OF_FIRE, 22, MON_SPELL_MAGICAL },
        { SPELL_HURL_DAMNATION, 22, MON_SPELL_MAGICAL },
-       { SPELL_FIRE_SUMMON, 22, MON_SPELL_MAGICAL },
+       { SPELL_HELLFIRE_COURT, 22, MON_SPELL_MAGICAL },
       }
     },
 
     {  MST_ANTAEUS,
       {
        { SPELL_LIGHTNING_BOLT, 24, MON_SPELL_MAGICAL },
-       { SPELL_FLASH_FREEZE, 48, MON_SPELL_MAGICAL },
+       { SPELL_SLEETSTRIKE, 36, MON_SPELL_MAGICAL },
       }
     },
 
@@ -2378,7 +2602,7 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_IRON_SHOT, 11, MON_SPELL_MAGICAL },
        { SPELL_FIRE_STORM, 11, MON_SPELL_MAGICAL },
-       { SPELL_FIRE_SUMMON, 11, MON_SPELL_MAGICAL },
+       { SPELL_HELLFIRE_COURT, 11, MON_SPELL_MAGICAL },
        { SPELL_HASTE, 36, MON_SPELL_MAGICAL },
       }
     },
