@@ -13,13 +13,13 @@
 
 #include "actor.h"
 #include "attribute-type.h"
+#include "bane-type.h"
 #include "beam.h"
 #include "bitary.h"
 #include "book-type.h"
 #include "caction-type.h"
 #include "daction-type.h"
 #include "duration-type.h"
-#include "equipment-type.h"
 #include "flush-reason-type.h"
 #include "game-chapter.h"
 #include "kill-method-type.h"
@@ -27,7 +27,9 @@
 #include "maybe-bool.h"
 #include "mon-holy-type.h"
 #include "mutation-type.h"
+#include "piety-info.h"
 #include "place-info.h"
+#include "player-equip.h"
 #include "quiver.h"
 #include "skill-menu-state.h"
 #include "species.h"
@@ -46,8 +48,7 @@
 #define FORCE_MAPPABLE_KEY "force_mappable"
 #define TEMP_WATERWALK_KEY "temp_waterwalk"
 #define EMERGENCY_FLIGHT_KEY "emergency_flight"
-#define PARALYSED_BY_KEY "paralysed_by"
-#define PETRIFIED_BY_KEY "petrified_by"
+#define DISABLED_BY_KEY "disabled_by"
 #define FROZEN_RAMPARTS_KEY "frozen_ramparts_position"
 #define DREAMSHARD_KEY "dreamshard"
 #define DESCENT_DEBT_KEY "descent_debt"
@@ -56,13 +57,30 @@
 #define RAMPAGE_HEAL_KEY "rampage_heal_strength"
 #define RAMPAGE_HEAL_MAX 7
 #define BLIND_COLOUR_KEY "blind_colour"
+#define TRICKSTER_POW_KEY "trickster_power"
+#define CACOPHONY_XP_KEY "cacophony_xp"
+#define BATFORM_XP_KEY "batform_xp"
+#define WATERY_GRAVE_XP_KEY "watery_grave_xp"
+#define WEREFURY_KEY "werefury_bonus"
+#define DEVIOUS_KEY "devious_stacks"
+#define FORCED_MESMERISE_KEY "forced_mesmerise"
+
+constexpr int ENKINDLE_CHARGE_COST = 40;
+#define ENKINDLE_CHARGES_KEY "enkindle_charges"
+#define ENKINDLE_PROGRESS_KEY "enkindle_progress"
+#define ENKINDLE_GIFT_GIVEN_KEY "enkindle_gifted"
+
+#define RIME_AURA_LAST_POS "rime_aura_pos"
+
+#define SOLAR_EMBER_MID_KEY "solar_ember_mid"
+#define SOLAR_EMBER_REVIVAL_KEY "solar_ember_revival"
+
+#define PYROMANIA_TRIGGERED_KEY "pyromania_triggered"
 
 // display/messaging breakpoints for penalties from Ru's MUT_HORROR
 #define HORROR_LVL_EXTREME  3
 #define HORROR_LVL_OVERWHELMING  5
 
-#define SEVERE_CONTAM_LEVEL 3
-
 /// Maximum stat value
 static const int MAX_STAT_VALUE = 125;
 /// The standard unit of regen; one level in artifact inscriptions
@@ -90,8 +108,6 @@ struct player_save_info;
 
 int player_stealth();
 
-enum class mutation_activity_type; // in mutation.h
-
 /// used for you.train[] & for rendering skill tiles (tileidx_skill)
 enum training_status
 {
@@ -104,6 +120,22 @@ enum training_status
     TRAINING_INACTIVE, ///< enabled but not used (in auto mode)
 };
 
+enum reprisal_type
+{
+    REPRISAL_HEADBUTT,  // Minotaur retaliatory headbutt
+    REPRISAL_FENCER,    // Fencer's Glove riposte
+};
+
+enum player_trigger_type
+{
+    DID_PARAGON,        // Platinum Paragon follow-up attack
+    DID_DITH_SHADOW,    // Dithmenos shadow mimic
+    DID_MEDUSA_STINGER, // Medusa form stinger attack
+    DID_SOLAR_EMBER,    // Sun scarab ember attack
+    DID_REV_UP,         // Coglin rev
+    NUM_PLAYER_TRIGGER_TYPES,
+};
+
 // needed for assert in is_player()
 #ifdef DEBUG_GLOBALS
 #define you (*real_you)
@@ -153,7 +185,6 @@ public:
     int max_magic_points;
     int mp_max_adj;             // max MP loss (ability costs, tutorial bonus)
 
-    FixedVector<int8_t, NUM_STATS> stat_loss;
     FixedVector<int8_t, NUM_STATS> base_stats;
 
     uint8_t max_level;
@@ -166,14 +197,8 @@ public:
     int gold;
     int zigs_completed, zig_max;
 
-    FixedVector<int8_t, NUM_EQUIP> equip;
-    FixedBitVector<NUM_EQUIP> melded;
-    // Whether these are unrands that we should run the _*_world_reacts func for
-    FixedBitVector<NUM_EQUIP> unrand_reacts;
-    // True if the slot has an item that activates when worn with max hp (regen
-    // items, acrobat amulet) and max hp has been reached while wearing it;
-    // false otherwise.
-    FixedBitVector<NUM_EQUIP> activated;
+    // Full set of infomation on which inventory items are equipped where.
+    player_equip_set equipment;
 
     FixedArray<int, NUM_OBJECT_CLASSES, MAX_SUBTYPES> force_autopickup;
 
@@ -181,7 +206,8 @@ public:
     monster_type symbol;
     transformation form;
     transformation default_form;
-    item_def active_talisman;
+    // Index into inv[] of the player's current talisman. (-1 if none.)
+    int8_t cur_talisman;
 
     // XXX: ENDOFPACK marks the total size of the player inventory, but we add
     //      a single extra slot after that for purposes of examining EV of
@@ -207,7 +233,6 @@ public:
     bool royal_jelly_dead;
     bool transform_uncancellable;
     bool fishtail; // Merfolk fishtail transformation
-    bool vampire_alive;
 
     unsigned short pet_target;
 
@@ -255,7 +280,6 @@ public:
     FixedVector<int, NUM_GODS> exp_docked_total; // XP-based wrath
 
     FixedArray<uint32_t, 6, MAX_SUBTYPES> item_description;
-    set<misc_item_type>                generated_misc;
     FixedVector<unique_item_status_type, MAX_UNRANDARTS> unique_items;
     uint8_t                            octopus_king_rings;
     unique_creature_list unique_creatures;
@@ -267,7 +291,7 @@ public:
 
     god_type religion;
     string jiyva_second_name;       // Random second name of Jiyva
-    uint8_t piety;
+    uint8_t raw_piety;
     uint8_t piety_hysteresis;       // amount of stored-up docking
     uint8_t gift_timeout;
     uint8_t saved_good_god_piety;   // for if you "switch" between E/Z/1 by abandoning one first
@@ -286,6 +310,8 @@ public:
 
     FixedVector<uint8_t, NUM_ABILITIES> sacrifice_piety;
 
+    FixedVector<int, NUM_BANES> banes;
+
     struct demon_trait
     {
         int           level_gained;
@@ -333,10 +359,17 @@ public:
     // All maps, by level.
     map<level_id, vector<string> > vault_list;
 
+    // The randomly chosen orb type to populate Zot with on this seed.
+    monster_type zot_orb_monster;
+    // And whether the player has gained knowledge of this by seeing a Zot statue.
+    bool zot_orb_monster_known;
+
     PlaceInfo global_info;
 
     LevelXPInfo global_xp_info;
 
+    PietyInfo piety_info;
+
     quiver::ammo_history m_quiver_history;
 
     quiver::action_cycler quiver_action;
@@ -386,7 +419,7 @@ public:
     // -------------------
     // Non-saved UI state:
     // -------------------
-    unsigned short prev_targ;
+    mid_t          prev_targ;
     coord_def      prev_grd_targ;
     // Examining spell library spells for Sif Muna's ability
     bool           divine_exegesis;
@@ -422,10 +455,16 @@ public:
     // ---------------------------
     bool turn_is_over; // player has performed a timed action
 
+    // In the middle of updating monsters upon returning to an explored floor.
+    bool doing_monster_catchup;
+
     // If true, player is headed to the Abyss.
     bool banished;
     string banished_by;
-    int banished_power;
+
+    // Position from which the player made an involuntary shout this turn.
+    // (To reduce message spam when encountering many monsters at once.)
+    coord_def shouted_pos;
 
     // If true, player has triggered a trap effect by exploring.
     bool trapped;
@@ -440,6 +479,8 @@ public:
     bool redraw_hit_points;
     bool redraw_magic_points;
     FixedVector<bool, NUM_STATS> redraw_stats;
+    bool redraw_doom;
+    bool redraw_contam;
     bool redraw_experience;
     bool redraw_armour_class;
     bool redraw_evasion;
@@ -466,6 +507,18 @@ public:
     mid_t damage_source; // death source of last damage done to player
     int source_damage; // cumulative damage for you.damage_source
 
+    // List of monsters the player has performed specific types of once-per-turn
+    // effects against.
+    vector<pair<mid_t, reprisal_type>> reprisals;
+
+    // List of triggered actions that can happen a limited number of times a turn.
+    FixedVector<int, NUM_PLAYER_TRIGGER_TYPES> triggers_done;
+
+    // Whether some form of attack was attempted at least once on the current
+    // turn (which can include failing due to fumbling in water, failing to
+    // reach past allies, or moving too fast for a martial attack to succeed).
+    bool attempted_attack;
+
     // When other levels are loaded (e.g. viewing), is the player on this level?
     bool on_current_level;
 
@@ -496,10 +549,11 @@ public:
 
     // Set player position without updating view geometry.
     void set_position(const coord_def &c) override;
-    // Low-level move the player. Use this instead of changing pos directly.
-    void moveto(const coord_def &c, bool clear_net = true) override;
-    bool move_to_pos(const coord_def &c, bool clear_net = true,
-                     bool /*force*/ = false) override;
+
+    bool move_to(const coord_def& newpos, movement_type flags = MV_DEFAULT,
+                 bool defer_finalisation = false) override;
+    void finalise_movement(const actor* to_blame = nullptr) override;
+
     // Move the player during an abyss shift.
     void shiftto(const coord_def &c);
     bool blink_to(const coord_def& c, bool quiet = false) override;
@@ -507,21 +561,19 @@ public:
     void set_level_visited(const level_id &level);
     bool level_visited(const level_id &level);
 
-    int stat(stat_type stat, bool nonneg = true) const;
+    int stat(stat_type stat, bool nonneg = true, bool innate_only = false) const;
     int strength(bool nonneg = true) const;
     int intel(bool nonneg = true) const;
     int dex(bool nonneg = true) const;
-    int max_stat(stat_type stat, bool base = false) const;
-    int max_strength() const;
-    int max_intel() const;
-    int max_dex() const;
+
+    int piety() const;
 
     bool in_water() const;
     bool in_liquid() const;
     bool can_swim(bool permanently = false) const;
     bool can_water_walk() const;
     int visible_igrd(const coord_def&) const;
-    bool rampaging() const override;
+    int rampaging() const override;
     bool is_banished() const override;
     bool is_sufficiently_rested(bool starting=false) const; // Up to rest_wait_percent HP and MP.
     bool is_web_immune() const override;
@@ -535,7 +587,7 @@ public:
     undead_state_type undead_state(bool temp = true) const;
     bool nightvision() const override;
     bool may_pruneify() const;
-    reach_type reach_range() const override;
+    int reach_range(bool include_weapon = true) const override;
     bool see_cell(const coord_def& p) const override;
 
     // Is c in view but behind a transparent wall?
@@ -550,15 +602,15 @@ public:
     bool spellcasting_unholy() const;
 
     // Dealing with beholders. Implemented in behold.cc.
-    void add_beholder(const monster& mon, bool axe = false);
+    void add_beholder(monster& mon, bool forced = false, int dur = 0);
     bool beheld() const;
     bool beheld_by(const monster& mon) const;
     monster* get_beholder(const coord_def &pos) const;
     monster* get_any_beholder() const;
-    void remove_beholder(const monster& mon);
+    void remove_beholder(monster& mon);
     void clear_beholders();
     void update_beholders();
-    void update_beholder(const monster* mon);
+    void update_beholder(monster* mon);
     bool possible_beholder(const monster* mon) const;
 
     // Dealing with fearmongers. Implemented in fearmonger.cc.
@@ -578,14 +630,13 @@ public:
     kill_category kill_alignment() const override;
 
     bool has_spell(spell_type spell) const override;
+    bool has_any_spells() const;
 
     string shout_verb(bool directed = false) const;
     int shout_volume() const;
 
-    item_def *slot_item(equipment_type eq, bool include_melded=false) const
-        override;
-
-    int base_ac_from(const item_def &armour, int scale = 1) const;
+    int base_ac_from(const item_def &armour, int scale = 1,
+                     bool include_form = true) const;
 
     int corrosion_amount() const;
 
@@ -610,27 +661,29 @@ public:
 
     god_type  deity() const override;
     bool      alive() const override;
-    bool      is_summoned(int* duration = nullptr,
-                          int* summon_type = nullptr) const override;
-    bool      is_perm_summoned() const override { return false; };
+    bool      alive_or_reviving() const override;
+    bool      is_summoned() const override { return false; };
+    bool      was_created_by(int) const override { return false; };
+    bool      was_created_by(const actor&, int = SPELL_NO_SPELL) const override
+                             { return false; };
+    bool      is_firewood() const override { return false; };
+    bool      is_peripheral() const override { return false; };
 
     bool        swimming() const override;
     bool        floundering() const override;
     bool        extra_balanced() const override;
     bool        slow_in_water() const;
-    bool        shove(const char* feat_name = "") override;
     bool        can_pass_through_feat(dungeon_feature_type grid) const override;
     bool        can_burrow() const override;
     bool        is_habitable_feat(dungeon_feature_type actual_grid) const
         override;
     size_type   body_size(size_part_type psize = PSIZE_TORSO,
                           bool base = false) const override;
-    brand_type  damage_brand(int which_attack = -1) override;
-    vorpal_damage_type damage_type(int which_attack = -1) override;
-    random_var  attack_delay(const item_def *projectile = nullptr,
-                             bool rescale = true) const override;
-    random_var  attack_delay_with(const item_def *projectile, bool rescale,
-                                  const item_def *weapon) const;
+    brand_type  damage_brand(const item_def* weapon) const;
+    vorpal_damage_type damage_type(const item_def* weapon) const;
+    random_var  attack_delay(const item_def *projectile = nullptr) const override;
+    random_var  melee_attack_delay() const override;
+    random_var  attack_delay_with(const item_def *weapon, bool melee_only = false) const;
     int         constriction_damage(constrict_type typ) const override;
 
     int       has_claws(bool allow_tran = true) const override;
@@ -650,8 +703,7 @@ public:
 
     // Information about player mutations. Implemented in mutation.cc
     int       get_base_mutation_level(mutation_type mut, bool innate=true, bool temp=true, bool normal=true) const;
-    int       get_mutation_level(mutation_type mut, bool check_form=true) const;
-    int       get_mutation_level(mutation_type mut, mutation_activity_type minact) const;
+    int       get_mutation_level(mutation_type mut, bool active_only=true) const;
     int       get_innate_mutation_level(mutation_type mut) const;
     int       get_temp_mutation_level(mutation_type mut) const;
 
@@ -662,36 +714,35 @@ public:
 
     bool      has_temporary_mutation(mutation_type mut) const;
     bool      has_innate_mutation(mutation_type mut) const;
-    bool      has_mutation(mutation_type mut, bool check_form=true) const;
+    bool      has_mutation(mutation_type mut, bool active_only=true) const;
+    bool      has_bane(bane_type bane) const;
 
-    int       how_mutated(bool innate=false, bool levels=false, bool temp=true) const;
+    bool      has_any_mutations() const;
+    int       how_mutated(bool normal = true, bool silver = false, bool all_innate = false,
+                          bool temp = false, bool levels = true) const;
 
-    int wearing(equipment_type slot, int sub_type) const
-        override;
-    int wearing_ego(equipment_type slot, int type) const
-        override;
+    int wearing(object_class_type obj_type, int sub_type,
+                bool count_plus = 0, bool check_attuned = false) const override;
+    int wearing_ego(object_class_type obj_type, int ego) const override;
     int scan_artefacts(artefact_prop_type which_property,
                        vector<const item_def *> *matches = nullptr) const override;
+    bool unrand_equipped(int unrand_index, bool include_melded = false) const override;
+
+    bool weapon_is_good_stab(const item_def *weapon = nullptr) const;
+    bool has_good_stab() const;
 
     int infusion_amount() const;
 
     item_def *weapon(int which_attack = -1) const override;
+    item_def *body_armour() const override;
     item_def *shield() const override;
+    item_def *offhand_item() const override;
     item_def *offhand_weapon() const override;
+    item_def *active_talisman() const;
 
     hands_reqd_type hands_reqd(const item_def &item,
                                bool base = false) const override;
 
-    bool      can_wield(const item_def &item,
-                        bool ignore_curse = false,
-                        bool ignore_brand = false,
-                        bool ignore_shield = false,
-                        bool ignore_transform = false) const override;
-    bool      could_wield(const item_def &item,
-                          bool ignore_brand = false,
-                          bool ignore_transform = false,
-                          bool quiet = true) const override;
-
     bool can_wear_barding(bool temp = false) const;
 
     string name(description_level_type type, bool force_visible = false,
@@ -724,18 +775,18 @@ public:
     bool has_bones(bool temp = true) const override;
     bool can_drink(bool temp = true) const;
     bool is_stationary() const override;
-    bool is_motile() const;
-    bool malmutate(const string &reason) override;
-    bool polymorph(int pow, bool allow_immobile = true) override;
+    bool malmutate(const actor* source, const string &reason = "") override;
+    bool polymorph(int dur) override;
+    bool doom(int amount) override;
     void backlight();
-    void banish(const actor* /*agent*/, const string &who = "", const int power = 0,
+    void banish(const actor* /*agent*/, const string &who = "",
                 bool force = false) override;
-    void blink() override;
+    void blink(bool ignore_stasis = false) override;
     void teleport(bool right_now = false,
                   bool wizard_tele = false) override;
-    void drain_stat(stat_type stat, int amount) override;
 
     void expose_to_element(beam_type element, int strength = 0,
+                           const actor* source = nullptr,
                            bool slow_cold_blood = true) override;
     void god_conduct(conduct_type thing_done, int level) override;
 
@@ -744,16 +795,24 @@ public:
     void paralyse(const actor *, int str, string source = "") override;
     void petrify(const actor *, bool force = false) override;
     bool fully_petrify(bool quiet = false) override;
+    bool vex(const actor* who, int dur, string source = "", string special_msg = "") override;
+    void give_stun_immunity(int duration);
     void slow_down(actor *, int str) override;
     void confuse(actor *, int strength) override;
     void weaken(const actor *attacker, int pow) override;
+    void diminish(const actor *attacker, int pow) override;
     bool strip_willpower(actor *attacker, int dur, bool quiet = false) override;
+    bool drain_magic(actor *attacker, int pow) override;
+    void daze(int duration) override;
+    void end_daze();
+    void vitrify(const actor *attacker, int duration, bool quiet = false) override;
+    bool floodify(const actor *attacker, int duration, const char* substance = "water") override;
     bool heal(int amount) override;
     bool drain(const actor *, bool quiet = false, int pow = 3) override;
     void splash_with_acid(actor *evildoer) override;
-    void acid_corrode(int acid_strength) override;
-    bool corrode_equipment(const char* corrosion_source = "the acid",
-                           int degree = 1) override;
+    bool corrode(const actor* source = nullptr,
+                 const char* corrosion_msg = "the acid",
+                 int amount = 4) override;
     void sentinel_mark(bool trap = false);
     int hurt(const actor *attacker, int amount,
              beam_type flavour = BEAM_MISSILE,
@@ -777,7 +836,8 @@ public:
     int how_chaotic(bool check_spells_god) const override;
     bool is_unbreathing() const override;
     bool is_insubstantial() const override;
-    int res_acid() const override;
+    bool is_amorphous() const override;
+    int res_corr() const override;
     bool res_damnation() const override { return false; };
     int res_fire() const override;
     int res_steam() const override;
@@ -794,17 +854,19 @@ public:
     bool res_polar_vortex() const override;
     bool res_petrify(bool temp = true) const override;
     bool res_constrict() const override;
+    int res_blind() const override;
     int willpower() const override;
     bool no_tele(bool blink = false, bool temp = true) const override;
     string no_tele_reason(bool blink = false, bool temp = true) const;
+    int slaying(bool throwing = false, bool random = true) const override;
     bool antimagic_susceptible() const override;
 
-    bool res_corr(bool allow_random = true, bool temp = true) const override;
     bool clarity(bool items = true) const override;
     bool faith(bool items = true) const override;
     bool reflection(bool items = true) const override;
     bool stasis() const override;
     bool cloud_immune(bool items = true) const override;
+    bool sunder_is_ready() const override;
 
     bool airborne() const override;
     bool permanent_flight(bool include_equip = true) const;
@@ -812,10 +874,20 @@ public:
     int get_noise_perception(bool adjusted = true) const;
     bool is_dragonkind() const override;
 
+    bool can_be_paralysed() const;
     bool paralysed() const override;
+    bool cannot_move() const override;
     bool cannot_act() const override;
+    bool helpless() const override;
     bool confused() const override;
+    bool is_silenced() const override;
+
     bool caught() const override;
+    void struggle_against_net() override;
+    bool trap_in_web() override;
+    bool trap_in_net(bool real, bool quiet = false) override;
+    void stop_being_caught(bool drop_net = false) override;
+
     bool backlit(bool self_halo = true, bool temp = true) const override;
     bool umbra() const override;
     int halo_radius() const override;
@@ -828,14 +900,13 @@ public:
     bool liquefied_ground() const override;
     bool incapacitated() const override
     {
-        return actor::incapacitated() || duration[DUR_CLUMSY];
+        return actor::incapacitated();
     }
     bool immune_to_hex(const spell_type hex) const;
 
     bool asleep() const override;
-    void put_to_sleep(actor *, int power = 0, bool hibernate = false) override;
-    void awaken();
-    void check_awaken(int disturbance) override;
+    void put_to_sleep(actor* source, int duration = 0, bool hibernate = false) override;
+    void wake_up(bool break_sleep = true, bool break_daze = true);
     int beam_resists(bolt &beam, int hurted, bool doEffects, string source)
         override;
     bool can_feel_fear(bool include_unknown) const override;
@@ -845,13 +916,10 @@ public:
     bool can_smell() const;
     bool can_sleep(bool holi_only = false) const override;
 
-    bool can_be_dazzled() const override;
-    bool can_be_blinded() const override;
-
     int racial_ac(bool temp) const;
     int base_ac(int scale) const;
     int armour_class() const override;
-    int gdr_perc() const override;
+    int gdr_perc(bool random = true) const override;
     int evasion(bool ignore_temporary = false,
                 const actor *attacker = nullptr) const override;
     int evasion_scaled(int scale, bool ignore_temporary = false,
@@ -865,25 +933,35 @@ public:
     int shield_bonus() const override;
     int shield_bypass_ability(int tohit) const override;
     void shield_block_succeeded(actor *attacker) override;
-    bool missile_repulsion() const override;
+    int missile_repulsion() const override;
 
     // Combat-related adjusted penalty calculation methods
-    int unadjusted_body_armour_penalty() const;
-    int adjusted_body_armour_penalty(int scale = 1) const;
+    int unadjusted_body_armour_penalty(bool archery = false) const;
+    int adjusted_body_armour_penalty(int scale = 1, bool archery = false) const;
     int adjusted_shield_penalty(int scale = 1) const;
 
-    // Calculates total permanent EV/SH if the player was/wasn't wearing a given item
-    void ac_ev_sh_with_specific_item(int scale, const item_def& new_item,
-                                     int *ac, int *ev, int *sh);
-    void ac_ev_sh_without_specific_item(int scale, const item_def& item_to_remove,
-                                        int *ac, int *ev, int *sh);
+    // Calculate the bonus (or penalty) the player has to their defenses from
+    // temporary effects.
+    int temp_ac_mod() const;
+    int temp_ev_mod() const;
+    int temp_sh_mod() const;
+
+    // Calculates total permanent AC/EV/SH if the player was/wasn't wearing a
+    // given item, along with the fail rate on all their known spells.
+    void preview_stats_with_specific_item(int scale, const item_def& new_item,
+                                          int *ac, int *ev, int *sh,
+                                          FixedVector<int, MAX_KNOWN_SPELLS> *fail);
+    void preview_stats_without_specific_item(int scale, const item_def& item_to_remove,
+                                             int *ac, int *ev, int *sh,
+                                             FixedVector<int, MAX_KNOWN_SPELLS> *fail);
+    void preview_stats_in_specific_form(int scale, const item_def& talisman,
+                                        int *ac, int *ev, int *sh,
+                                        FixedVector<int, MAX_KNOWN_SPELLS> *fail);
 
     bool wearing_light_armour(bool with_skill = false) const;
     int  skill(skill_type skill, int scale = 1, bool real = false,
                bool temp = true) const override;
 
-    bool using_talisman(const item_def &talisman) const;
-
     bool do_shaft() override;
     bool shaftable() const;
 
@@ -894,12 +972,6 @@ public:
     int scale_potion_healing(int healing_amount);
     int scale_potion_mp_healing(int healing_amount);
 
-    void apply_location_effects(const coord_def &oldpos,
-                                killer_type killer = KILL_NONE,
-                                int killernum = -1) override;
-
-    void did_deliberate_movement() override;
-
     void be_agile(int pow);
 
     int rev_percent() const;
@@ -909,6 +981,11 @@ public:
 
     bool allies_forbidden();
 
+    void track_reprisal(reprisal_type type, mid_t target_mid);
+    bool did_reprisal(reprisal_type type, mid_t target_mid);
+
+    void did_trigger(player_trigger_type trigger);
+
     // TODO: move this somewhere else
     void refresh_rampage_hints();
 
@@ -950,18 +1027,11 @@ public:
 
     bool form_uses_xl() const;
 
-    bool clear_far_engulf(bool force = false, bool moved = false) override;
+    virtual void clear_constricted() override;
 
     int armour_class_scaled(int scale) const;
 
     int ac_changes_from_mutations() const;
-    vector<const item_def *> get_armour_items() const;
-    vector<const item_def *> get_armour_items_one_sub(const item_def& sub) const;
-    vector<const item_def *> get_armour_items_one_removal(const item_def& sub) const;
-    int base_ac_with_specific_items(int scale,
-                                    vector<const item_def *> armour_items) const;
-    int armour_class_with_specific_items(int scale,
-                                vector<const item_def *> items) const;
 
 protected:
     void _removed_beholder(bool quiet = false);
@@ -969,7 +1039,6 @@ protected:
 
     void _removed_fearmonger(bool quiet = false);
     bool _possible_fearmonger(const monster* mon) const;
-
 };
 
 class monster;
@@ -978,16 +1047,11 @@ struct item_def;
 class player_vanishes
 {
     coord_def source;
-    bool movement;
 public:
-    player_vanishes(bool _movement=false);
+    player_vanishes();
     ~player_vanishes();
 };
 
-// Helper. Use move_player_to_grid or player::apply_location_effects instead.
-void moveto_location_effects(dungeon_feature_type old_feat,
-                             bool stepped=false, const coord_def& old_pos=coord_def());
-
 bool check_moveto(const coord_def& p, const string &move_verb = "step",
                   bool physically = true);
 bool check_moveto_terrain(const coord_def& p, const string &move_verb,
@@ -1003,6 +1067,8 @@ bool check_moveto_exclusion(const coord_def& p,
 bool check_moveto_trap(const coord_def& p, const string &move_verb = "step",
         bool *prompted = nullptr);
 
+bool check_move_over(coord_def p, const string& move_verb);
+
 bool swap_check(monster* mons, coord_def &loc, bool quiet = false);
 
 void move_player_to_grid(const coord_def& p, bool stepped);
@@ -1017,28 +1083,30 @@ static inline bool player_in_branch(int branch)
 }
 
 bool berserk_check_wielded_weapon();
-bool player_equip_unrand(int unrand_index, bool include_melded = false);
 bool player_can_hear(const coord_def& p, int hear_distance = 999);
 
-bool player_is_shapechanged();
-
 void update_acrobat_status();
 bool player_acrobatic();
 
+int player_parrying();
+void update_parrying_status();
+
 bool is_effectively_light_armour(const item_def *item);
 bool player_effectively_in_light_armour();
 
 int player_shield_racial_factor();
 int player_armour_shield_spell_penalty();
+int player_armour_stealth_penalty();
 
 int player_movement_speed(bool check_terrain = true, bool temp = true);
 
 int player_icemail_armour_class();
 int player_condensation_shield_class();
 int sanguine_armour_bonus();
+int stone_body_armour_bonus();
 
 int player_wizardry();
-int player_channelling();
+int player_channelling_chance(bool max = false);
 
 int player_prot_life(bool allow_random = true, bool temp = true,
                      bool items = true);
@@ -1053,7 +1121,6 @@ bool player_likes_water(bool permanently = false);
 
 int player_res_cold(bool allow_random = true, bool temp = true,
                     bool items = true);
-int player_res_acid(bool items = true);
 int player_res_electricity(bool allow_random = true, bool temp = true,
                            bool items = true);
 int player_res_fire(bool allow_random = true, bool temp = true,
@@ -1063,6 +1130,8 @@ int player_res_steam(bool allow_random = true, bool temp = true,
                      bool items = true);
 int player_res_poison(bool allow_random = true, bool temp = true,
                       bool items = true, bool forms = true);
+int player_res_corrosion(bool allow_random = true, bool temp = true,
+                         bool items = true);
 int player_willpower(bool temp = true);
 
 int player_shield_class(int scale = 1, bool random = true,
@@ -1079,6 +1148,7 @@ int player_spec_fire();
 int player_spec_hex();
 int player_spec_alchemy();
 int player_spec_summ();
+int player_spec_forgecraft();
 int player_spec_tloc();
 
 int player_speed();
@@ -1090,8 +1160,6 @@ int get_teleportitis_level();
 
 int player_monster_detect_radius();
 
-int slaying_bonus(bool throwing = false, bool random = true);
-
 unsigned int exp_needed(int lev, int exp_apt = -99);
 bool will_gain_life(int lev);
 
@@ -1114,7 +1182,7 @@ monster_type player_mons(bool transform = true);
 void update_player_symbol();
 void update_vision_range();
 
-maybe_bool you_can_wear(equipment_type eq, bool temp = false);
+maybe_bool you_can_wear(equipment_slot slot, bool include_form = false);
 bool player_can_use_armour();
 
 bool player_has_hair(bool temp = true, bool include_mutations = true);
@@ -1152,9 +1220,9 @@ void set_hp(int new_amount);
 int get_real_hp(bool trans, bool drained = true);
 int get_real_mp(bool include_items);
 
-int get_contamination_level();
-bool player_severe_contamination();
-string describe_contamination(int level);
+bool player_harmful_contamination();
+int contam_max_damage();
+string describe_contamination(bool verbose = true);
 
 bool sanguine_armour_valid();
 void activate_sanguine_armour();
@@ -1175,7 +1243,6 @@ bool confuse_player(int amount, bool quiet = false, bool force = false);
 
 bool poison_player(int amount, string source, string source_aux = "",
                    bool force = false);
-void paralyse_player(string source);
 void handle_player_poison(int delay);
 void reduce_player_poison(int amount);
 int get_player_poisoning();
@@ -1189,6 +1256,10 @@ void dec_sticky_flame_player(int delay);
 void shake_off_sticky_flame();
 void end_sticky_flame_player();
 
+void silence_player(int turns);
+
+const char* player_silenced_reason();
+
 bool spell_slow_player(int pow);
 bool slow_player(int turns);
 void dec_slow_player(int delay);
@@ -1204,7 +1275,11 @@ void dec_ambrosia_player(int delay);
 void dec_channel_player(int delay);
 void dec_frozen_ramparts(int delay);
 void reset_rampage_heal_duration();
-void apply_rampage_heal();
+void apply_rampage_heal(int distance_moved);
+void trickster_trigger(const monster& victim, enchant_type ench);
+int trickster_bonus();
+int enkindle_max_charges();
+void maybe_harvest_memory(const monster& victim);
 bool invis_allowed(bool quiet = false, string *fail_reason = nullptr,
                                                         bool temp = true);
 bool flight_allowed(bool quiet = false, string *fail_reason = nullptr);
@@ -1223,7 +1298,6 @@ bool is_feat_dangerous(dungeon_feature_type feat, bool permanently = false,
                        bool ignore_flight = false);
 void enable_emergency_flight();
 
-int count_worn_ego(int which_ego);
 bool need_expiration_warning(duration_type dur, dungeon_feature_type feat);
 bool need_expiration_warning(dungeon_feature_type feat);
 bool need_expiration_warning(duration_type dur, coord_def p = you.pos());
