@@ -45,6 +45,7 @@ struct game_state
     bool mouse_enabled;     // True if mouse input is currently relevant.
 
     bool waiting_for_command; // True when the game is waiting for a command.
+    bool waiting_for_ui;     // True when waiting for input from the ui overlay
     bool terminal_resized;   // True if the term was resized and we need to
                              // take action to handle it.
     time_t last_winch;       // Time of last resize, for crash dumps.
@@ -86,6 +87,7 @@ struct game_state
     bool test_list;         // Show available tests and exit.
     bool script;            // Set if we want to run a Lua script and exit.
     bool build_db;          // Set if we want to rebuild the db and exit.
+    bool use_des_cache;
     vector<string> tests_selected; // Tests to be run.
     vector<string> script_args;    // Arguments to scripts.
 
@@ -101,6 +103,8 @@ struct game_state
 
     bool show_more_prompt;  // Set to false to disable --more-- prompts.
 
+    bool skip_autofight_check; // XXX EVIL HACK
+
     string sprint_map;      // Sprint map set on command line, if any.
 
     string map;             // Map selected in the newgame menu
@@ -133,14 +137,15 @@ struct game_state
 
     bool invisible_targeting;
 
-    bool player_moving;
-
     // Area beyond which view should be darkened,  0 = disabled.
     targeter *darken_range;
 
     // Monsters to highlight on the screen, 0 = disabled.
     vector<monster *> *flash_monsters;
 
+    // monsters which saw the player retreating.
+    set<monster*> potential_pursuers;
+
     // Any changes to macros that need to be changed?
     bool unsaved_macros;
 
@@ -219,7 +224,6 @@ public:
     void dump();
     bool player_is_dead() const;
 
-    bool game_standard_levelgen() const;
     bool game_is_valid_type() const;
     bool game_is_normal() const;
     bool game_is_tutorial() const;
@@ -227,6 +231,10 @@ public:
     bool game_is_sprint() const;
     bool game_is_hints() const;
     bool game_is_hints_tutorial() const;
+    bool game_is_descent() const;
+
+    bool game_has_random_floors() const;
+    bool game_saves_prefs() const;
 
     bool seed_is_known() const;
 
