@@ -21,8 +21,11 @@
 #include "files.h"
 #include "libutil.h"
 #include "maps.h"
+#include "map-knowledge.h"
+#include "mon-act.h"
 #include "message.h"
 #include "place.h"
+#include "player-notices.h"
 #include "prompt.h"
 #include "religion.h"
 #include "spl-goditem.h" // detect_items
@@ -111,6 +114,12 @@ void wizard_level_travel(bool down)
 
     if (down)
         down_stairs(stairs, false, false);
+    else if (stairs == DNGN_EXIT_VAULTS && vaults_is_locked())
+    {
+        unlock_vaults();
+        up_stairs(stairs, false);
+        lock_vaults();
+    }
     else
         up_stairs(stairs, false);
 
@@ -155,8 +164,7 @@ static void _wizard_go_to_level(const level_pos &pos)
     _wizard_level_target = pos.id;
 
     leaving_level_now(stair_taken);
-    const bool newlevel = load_level(stair_taken, LOAD_ENTER_LEVEL, old_level);
-    tile_new_level(newlevel);
+    load_level(stair_taken, LOAD_ENTER_LEVEL, old_level);
     if (!crawl_state.test)
         save_game_state();
     new_level();
@@ -270,6 +278,12 @@ bool wizard_create_feature(const coord_def& pos, dungeon_feature_type feat, bool
     return wizard_create_feature(t, feat, mimic);
 }
 
+static void _connect_door(coord_def pos)
+{
+    if (map_bounds(pos) && feat_is_door(env.grid(pos)))
+        tile_init_flavour(pos);
+}
+
 bool wizard_create_feature(dist &target, dungeon_feature_type feat, bool mimic)
 {
     if (feat == DNGN_UNSEEN)
@@ -310,35 +324,47 @@ bool wizard_create_feature(dist &target, dungeon_feature_type feat, bool mimic)
         }
         coord_def &pos = target.target;
 
+        bool done = false;
+        bool success = false;
         if (feat == DNGN_ENTER_SHOP)
-            return debug_make_shop(pos);
-
-        if (feat_is_trap(feat))
-            return debug_make_trap(pos);
-
-        tile_env.flv(pos).feat = 0;
-        tile_env.flv(pos).special = 0;
-        env.grid_colours(pos) = 0;
-        const dungeon_feature_type old_feat = env.grid(pos);
-        dungeon_terrain_changed(pos, feat, false, false, false, true);
-        // Update gate tiles, if existing.
-        if (feat_is_door(old_feat) || feat_is_door(feat))
         {
-            const coord_def left  = pos - coord_def(1, 0);
-            const coord_def right = pos + coord_def(1, 0);
-            if (map_bounds(left) && feat_is_door(env.grid(left)))
-                tile_init_flavour(left);
-            if (map_bounds(right) && feat_is_door(env.grid(right)))
-                tile_init_flavour(right);
+            success = debug_make_shop(pos);
+            done = true;
+        }
+        else if (feat_is_trap(feat))
+        {
+            success = debug_make_trap(pos, trap_type_from_feature(feat));
+            done = true;
+        }
+        else
+        {
+            tile_env.flv(pos).feat = 0;
+            tile_env.flv(pos).special = 0;
+            env.grid_colours(pos) = 0;
+            const dungeon_feature_type old_feat = env.grid(pos);
+            dungeon_terrain_changed(pos, feat, false, false, true);
+            // Update gate tiles, if existing.
+            if (feat_is_door(old_feat) || feat_is_door(feat))
+            {
+                _connect_door(pos - coord_def(1, 0));
+                _connect_door(pos + coord_def(1, 0));
+                _connect_door(pos - coord_def(0, 1));
+                _connect_door(pos + coord_def(0, 1));
+            }
+            if (pos == you.pos() && cell_is_solid(pos))
+                you.wizmode_teleported_into_rock = true;
         }
-        if (pos == you.pos() && cell_is_solid(pos))
-            you.wizmode_teleported_into_rock = true;
 
         if (mimic)
             env.level_map_mask(pos) |= MMT_MIMIC;
 
         if (you.see_cell(pos))
+        {
             view_update_at(pos);
+            StashTrack.update_stash(pos);
+        }
+        if (done)
+            return success;
     } while (targeting_mode && target.isEndpoint);
 
     return true;
@@ -429,7 +455,7 @@ void wizard_map_level()
     for (rectangle_iterator ri(BOUNDARY_BORDER - 1); ri; ++ri)
     {
         update_item_at(*ri, true);
-        show_update_at(*ri, LAYER_ITEMS);
+        show_update_at(*ri, Layer::ITEMS);
 
 #ifdef USE_TILE
         tiles.update_minimap(*ri);
@@ -441,74 +467,38 @@ void wizard_map_level()
     }
 }
 
-bool debug_make_trap(const coord_def& pos)
+bool debug_make_trap(const coord_def& pos, trap_type trap)
 {
-    char requested_trap[80];
-    trap_type trap = TRAP_UNASSIGNED;
-    int gridch     = env.grid(pos);
-
-    if (gridch != DNGN_FLOOR)
+    if (env.grid(pos) != DNGN_FLOOR)
     {
-        mpr("You need to be on a floor square to make a trap.");
+        mprf("You can only make a %s on a floor square.",
+             trap == TRAP_UNASSIGNED ? "trap" : full_trap_name(trap).c_str());
         return false;
     }
 
-    msgwin_get_line("What kind of trap? ",
-                    requested_trap, sizeof(requested_trap));
-    if (!*requested_trap)
-    {
-        canned_msg(MSG_OK);
-        return false;
-    }
-
-    string spec = lowercase_string(requested_trap);
-    vector<trap_type> matches;
-    vector<string>    match_names;
-
-    if (spec == "random" || spec == "any")
-        trap = TRAP_RANDOM;
-
-    for (int t = TRAP_FIRST_TRAP; t < NUM_TRAPS; ++t)
-    {
-        const trap_type tr = static_cast<trap_type>(t);
-        string tname       = lowercase_string(trap_name(tr));
-        if (spec.find(tname) != spec.npos)
-        {
-            trap = tr;
-            break;
-        }
-        else if (tname.find(spec) != tname.npos)
-        {
-            matches.push_back(tr);
-            match_names.push_back(tname);
-        }
-    }
-
     if (trap == TRAP_UNASSIGNED)
     {
-        if (matches.empty())
+        vector<WizardEntry> options;
+        for (int i = TRAP_FIRST_TRAP; i < NUM_TRAPS; ++i)
         {
-            mprf("I know no traps named \"%s\".", spec.c_str());
-            return false;
+            auto name = trap_name(static_cast<trap_type>(i));
+            options.emplace_back(WizardEntry(name, i));
         }
-        // Only one match, use that
-        else if (matches.size() == 1)
-            trap = matches[0];
-        else
-        {
-            string prefix = "No exact match for trap '";
-            prefix += spec;
-            prefix += "', possible matches are: ";
-            mpr_comma_separated_list(prefix, match_names);
+        sort(options.begin(), options.end());
+        options.emplace_back(WizardEntry('*', "any", TRAP_RANDOM));
+
+        auto menu = WizardMenu("Make which kind of trap?", options);
+        if (!menu.run(true))
             return false;
-        }
+
+        trap = static_cast<trap_type>(menu.result());
     }
+    place_specific_trap(pos, trap);
 
-    place_specific_trap(you.pos(), trap);
     mprf("Created %s.",
          (trap == TRAP_RANDOM)
             ? "a random trap"
-            : trap_at(you.pos())->name(DESC_A).c_str());
+            : trap_at(pos)->name(DESC_A).c_str());
 
     if (trap == TRAP_SHAFT && !is_valid_shaft_level())
         mpr("NOTE: Shaft traps aren't valid on this level.");
@@ -524,25 +514,21 @@ bool debug_make_shop(const coord_def& pos)
         return false;
     }
 
-    char requested_shop[80];
-    msgwin_get_line("What kind of shop? ",
-                    requested_shop, sizeof(requested_shop));
-    if (!*requested_shop)
+    vector<WizardEntry> options;
+    for (int i = 0; i < NUM_SHOPS; ++i)
     {
-        canned_msg(MSG_OK);
-        return false;
+        auto name = shoptype_to_str(static_cast<shop_type>(i));
+        if (str_to_shoptype(name) == i) // Don't offer to make "removed" shops.
+            options.emplace_back(WizardEntry(name, i));
     }
+    sort(options.begin(), options.end());
+    options.emplace_back(WizardEntry('*', "any", SHOP_RANDOM));
 
-    const shop_type new_shop_type = str_to_shoptype(requested_shop);
-
-    if (new_shop_type == SHOP_UNASSIGNED)
-    {
-        mprf("Bad shop type: \"%s\"", requested_shop);
-        list_shop_types();
+    auto menu = WizardMenu("Make which kind of shop?", options);
+    if (!menu.run(true))
         return false;
-    }
 
-    place_spec_shop(pos, new_shop_type);
+    place_spec_shop(pos, static_cast<shop_type>(menu.result()));
     mpr("Done.");
     return true;
 }
@@ -637,8 +623,8 @@ static void debug_load_map_by_name(string name, bool primary)
                        "for this location; placing it with &P may result in "
                        "crashes and save corruption. Continue?", true, 'y'))
         {
-            mprf("Ok; try placing with &L or go to the relevant location to "
-                 "safely place with &P.");
+            mpr("Ok; try placing with &L or go to the relevant location to "
+                "safely place with &P.");
             return;
         }
         if (toplace->is_minivault())
@@ -776,7 +762,7 @@ void debug_test_explore()
     const int explore_turns = _debug_time_explore();
 
     // Return to starting point.
-    you.moveto(where);
+    you.move_to(where, MV_INTERNAL);
 
     mprf("Explore took %d turns.", explore_turns);
 }
@@ -855,11 +841,10 @@ void wizard_recreate_level()
 
     leaving_level_now(stair_taken);
     delete_level(lev);
-    const bool newlevel = load_level(stair_taken, LOAD_START_GAME, lev);
+    load_level(stair_taken, LOAD_START_GAME, lev);
     if (you.get_place_info().levels_seen > 1)
         you.get_place_info().levels_seen--; // this getting to 0 -> crashes
 
-    tile_new_level(newlevel);
     if (!crawl_state.test)
         save_game_state();
     new_level();
@@ -878,6 +863,8 @@ void wizard_clear_used_vaults()
     you.uniq_map_names_abyss.clear();
     env.level_uniq_maps.clear();
     env.level_uniq_map_tags.clear();
+    for (int i = 0; i < NUM_BRANCHES; ++i)
+        branch_uniq_map_tags[i].clear();
     mpr("All vaults are now eligible for [re]use.");
 }
 
